searchData={"items":[{"type":"module","title":"Magma","doc":"Magma is an environment for writing and executing complex prompts.\n\nIt is primarily designed to support developers in documenting their projects.\nIt provides a system of documents for predefined workflows, to generate\nvarious documentation artefacts.\n\nRead the [User Guide](Magma User Guide - Introduction to Magma (article section).md) to learn more.","ref":"Magma.html"},{"type":"macro","title":"Magma.defmoduledoc/0","doc":"Adds the contents of the final version of the `Magma.Artefacts.ModuleDoc` as the `@moduledoc`.\n\nUsually this done via `use Magma`.\n\n> ##","ref":"Magma.html#defmoduledoc/0"},{"type":"macro","title":"warning {: .warning} - Magma.defmoduledoc/0","doc":">\n> If you decide to include your moduledocs with this macro, be aware that if\n> you're writing a library and your users should be able to use these docs on\n> their machines, e.g. with the `h` helper in IEx you'll have to include the\n> Magma documents with the final moduledocs in your package like this:\n>\n> ```elixir\n> defp package do\n>   [\n>     # ...\n>     files:  ~w[lib priv mix.exs docs.magma/artefacts/final/modules/**/*.md]\n>   ]\n> end\n> ```","ref":"Magma.html#defmoduledoc/0-warning-warning"},{"type":"function","title":"Magma.version/0","doc":"","ref":"Magma.html#version/0"},{"type":"module","title":"Magma.View","doc":"Utility module with helper functions for creating the content of Magma documents.","ref":"Magma.View.html"},{"type":"function","title":"Magma.View.button/3","doc":"","ref":"Magma.View.html#button/3"},{"type":"function","title":"Magma.View.callout/2","doc":"","ref":"Magma.View.html#callout/2"},{"type":"function","title":"Magma.View.comment/1","doc":"","ref":"Magma.View.html#comment/1"},{"type":"function","title":"Magma.View.delete_current_file_button/0","doc":"","ref":"Magma.View.html#delete_current_file_button/0"},{"type":"function","title":"Magma.View.include/3","doc":"","ref":"Magma.View.html#include/3"},{"type":"function","title":"Magma.View.include_context_knowledge/1","doc":"","ref":"Magma.View.html#include_context_knowledge/1"},{"type":"function","title":"Magma.View.link_to/2","doc":"Creates an internal Obsidian link to the given document.\n\nThe document can be given by name or as a `Magma.Document` struct.\n\nThe optional second argument allows set an alternative label.","ref":"Magma.View.html#link_to/2"},{"type":"function","title":"Examples - Magma.View.link_to/2","doc":"iex> Magma.View.link_to(\"Document\")\n    \"[[Document]]\"\n\n    iex> Magma.View.link_to(\"Document\", \"Alt\")\n    \"[[Document|Alt]]\"","ref":"Magma.View.html#link_to/2-examples"},{"type":"function","title":"Magma.View.link_to_concept/2","doc":"","ref":"Magma.View.html#link_to_concept/2"},{"type":"function","title":"Magma.View.link_to_preview/2","doc":"","ref":"Magma.View.html#link_to_preview/2"},{"type":"function","title":"Magma.View.link_to_prompt/2","doc":"","ref":"Magma.View.html#link_to_prompt/2"},{"type":"function","title":"Magma.View.link_to_prompt_result/2","doc":"","ref":"Magma.View.html#link_to_prompt_result/2"},{"type":"function","title":"Magma.View.link_to_version/2","doc":"","ref":"Magma.View.html#link_to_version/2"},{"type":"function","title":"Magma.View.prompt_results_table/0","doc":"","ref":"Magma.View.html#prompt_results_table/0"},{"type":"function","title":"Magma.View.transclude/2","doc":"Creates an Obsidian transclusion of the given document or section.\n\nThe document can be given by name or as a `Magma.Document` struct.\n\nThe optional second argument can be used to specify a specific\nsection to be transcluded.","ref":"Magma.View.html#transclude/2"},{"type":"function","title":"Examples - Magma.View.transclude/2","doc":"iex> Magma.View.transclude(\"Document\")\n    \"![[Document|]]\"\n\n    iex> Magma.View.transclude(\"Document\", \"Section\")\n    \"![[Document#Section|]]\"","ref":"Magma.View.html#transclude/2-examples"},{"type":"function","title":"Magma.View.transclude_concept/2","doc":"","ref":"Magma.View.html#transclude_concept/2"},{"type":"function","title":"Magma.View.transclude_preview/2","doc":"","ref":"Magma.View.html#transclude_preview/2"},{"type":"function","title":"Magma.View.transclude_prompt/2","doc":"","ref":"Magma.View.html#transclude_prompt/2"},{"type":"function","title":"Magma.View.transclude_prompt_result/2","doc":"","ref":"Magma.View.html#transclude_prompt_result/2"},{"type":"function","title":"Magma.View.transclude_version/2","doc":"","ref":"Magma.View.html#transclude_version/2"},{"type":"function","title":"Magma.View.yaml_list/1","doc":"","ref":"Magma.View.html#yaml_list/1"},{"type":"function","title":"Magma.View.yaml_nested_map/1","doc":"","ref":"Magma.View.html#yaml_nested_map/1"},{"type":"task","title":"mix magma.artefact.select_draft","doc":"A Mix task for selecting a prompt result as a draft version.\n\n``` sh\n$ mix magma.artefact.select_draft \"Name of prompt result\" \n```","ref":"Mix.Tasks.Magma.Artefact.SelectDraft.html"},{"type":"function","title":"Mix.Tasks.Magma.Artefact.SelectDraft.run/1","doc":"","ref":"Mix.Tasks.Magma.Artefact.SelectDraft.html#run/1"},{"type":"task","title":"mix magma.prompt.copy","doc":"A Mix task for rendering a specified prompt, resolving all transclusions, and copying the result to the clipboard.\n\n``` sh\n$ mix magma.prompt.copy \"Name of prompt\"\n```","ref":"Mix.Tasks.Magma.Prompt.Copy.html"},{"type":"function","title":"Mix.Tasks.Magma.Prompt.Copy.run/1","doc":"","ref":"Mix.Tasks.Magma.Prompt.Copy.html#run/1"},{"type":"task","title":"mix magma.prompt.exec","doc":"A Mix task for executing prompts.\n\nThis Mix task facilitates the execution of a given prompt either manually or according to the generation specification embedded within the prompt document, depending on the `magma_generation_type` and `magma_generation_params` properties specified in the YAML frontmatter.\n\nThe task can be invoked with a prompt name or path, like so:\n\n``` sh\n$ mix magma.prompt.exec \"Name of prompt\"\n```\n\nUsing the `--manual` switch allows for manual execution of the prompt. In this mode, the rendered prompt is copied to the clipboard, ready for pasting and executing in the LLM interface of one's choice. By default, the user is interactively asked to paste the result of this execution, which is then used to create a prompt result document:\n\n``` sh\n$ mix magma.prompt.exec \"Name of prompt\" --manual\n```\n\nThe `--no-interactive` switch disables the interactive prompt for pasting the result back and instead creates an empty prompt result document. This is useful in contexts where shell interaction is not possible, such as with Obsidian buttons.\n\n#","ref":"Mix.Tasks.Magma.Prompt.Exec.html"},{"type":"task","title":"Configuration - mix magma.prompt.exec","doc":"The default values for the generation specification embedded within the prompt document (the `magma_generation_type` and `magma_generation_params` properties in its YAML frontmatter) can be configured for your application in `config.exs` like this\n\n``` elixir\nconfig :magma,  \n  default_generation: Magma.Generation.OpenAI\n  \nconfig :magma, Magma.Generation.OpenAI,  \n  model: \"gpt-4\",  \n  temperature: 0.6\n```\n\n#","ref":"Mix.Tasks.Magma.Prompt.Exec.html#module-configuration"},{"type":"task","title":"Command line options - mix magma.prompt.exec","doc":"-   `--manual` - Executes the prompt manually, copying the rendered prompt to the clipboard for pasting and executing in an LLM interface\n-   `--no-interactive` - Disables the interactive prompt for pasting the result back and creates an empty prompt result document instead","ref":"Mix.Tasks.Magma.Prompt.Exec.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Magma.Prompt.Exec.run/1","doc":"","ref":"Mix.Tasks.Magma.Prompt.Exec.html#run/1"},{"type":"task","title":"mix magma.prompt.gen","doc":"A Mix task for generating Magma artefact prompt documents and custom prompt documents.\n\nThis Mix task is used to create either a custom Magma prompt document or an artefact prompt document. The task requires different arguments depending on the type of document to be created.\n\nFor a custom prompt document, a single argument representing the name of the prompt is required.\n\n``` sh\n$ mix magma.prompt.gen \"Prompt for something\"\n```\n\nFor an artefact prompt document, two arguments are needed: the first being the name of a concept and the second being the artefact type. The artefact type should be the last part of an `Magma.Artefacts.X` artefact type module, e.g. `ModuleDoc` or `Readme`.\n\n``` sh\n$ mix magma.prompt.gen \"Some.Module\" ModuleDoc\n```\n\nNote that, by default, all artefact prompts of a concept (according to its matter type) are already created when a concept document is created. Therefore, it's usually not necessary to use this task for generating artefact prompt documents. If you only want to update an existing artefact prompt, you can use the `Mix.Tasks.Magma.Prompt.Update` Mix task instead.\n\n#","ref":"Mix.Tasks.Magma.Prompt.Gen.html"},{"type":"task","title":"Command line options - mix magma.prompt.gen","doc":"-   `--force` - When set, this option allows the task to overwrite existing documents without asking for permission first.","ref":"Mix.Tasks.Magma.Prompt.Gen.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Magma.Prompt.Gen.run/1","doc":"","ref":"Mix.Tasks.Magma.Prompt.Gen.html#run/1"},{"type":"task","title":"mix magma.prompt.update","doc":"A Mix task for regenerating artefact prompt documents.\n\nThis task is useful for example when an artefact prompt contains the code of a module that has been modified since the creation of the prompt document. By regenerating the artefact prompt, the updated code is reflected in the prompt document.\n\n``` sh\n$ mix magma.prompt.update \"Name of a Prompt\"\n```\n\n#","ref":"Mix.Tasks.Magma.Prompt.Update.html"},{"type":"task","title":"Command line options - mix magma.prompt.update","doc":"-   `--all` - When set, all prompts of the vault are updated.","ref":"Mix.Tasks.Magma.Prompt.Update.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Magma.Prompt.Update.all_prompt_files/1","doc":"","ref":"Mix.Tasks.Magma.Prompt.Update.html#all_prompt_files/1"},{"type":"function","title":"Mix.Tasks.Magma.Prompt.Update.run/1","doc":"","ref":"Mix.Tasks.Magma.Prompt.Update.html#run/1"},{"type":"function","title":"Mix.Tasks.Magma.Prompt.Update.update/1","doc":"","ref":"Mix.Tasks.Magma.Prompt.Update.html#update/1"},{"type":"function","title":"Mix.Tasks.Magma.Prompt.Update.update_all/0","doc":"","ref":"Mix.Tasks.Magma.Prompt.Update.html#update_all/0"},{"type":"task","title":"mix magma.text.assemble","doc":"A Mix task for generating the section documents of a text from its final table of contents.\n\nThis task is used once the final artefact version document for the `Magma.Artefacts.TableOfContents` of a text has been created. This task will then create the concept and artefact prompt documents of the sections of the text and assemble the preview document.\n\n``` sh\nmix magma.text.assemble \"Name of ToC document\"\n```\n\n#","ref":"Mix.Tasks.Magma.Text.Assemble.html"},{"type":"task","title":"Command line options - mix magma.text.assemble","doc":"-   `--force` - Allows the task to overwrite existing documents without asking for permission first.","ref":"Mix.Tasks.Magma.Text.Assemble.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Magma.Text.Assemble.run/1","doc":"","ref":"Mix.Tasks.Magma.Text.Assemble.html#run/1"},{"type":"task","title":"mix magma.text.finalize","doc":"A Mix task for generating the final text from a given preview document.\n\n``` sh\nmix magma.text.finalize \"Name of preview document\"\n```","ref":"Mix.Tasks.Magma.Text.Finalize.html"},{"type":"function","title":"Mix.Tasks.Magma.Text.Finalize.run/1","doc":"","ref":"Mix.Tasks.Magma.Text.Finalize.html#run/1"},{"type":"task","title":"mix magma.text.new","doc":"A Mix task for generating the concept and artefact prompt documents for a new text.\n\nThe first argument is the title of your text, followed by an optional text type. The text type corresponds to the last part of the available text type modules of the form `Magma.Matter.Texts.X`. The text type determines the details of the system prompt of the artefact prompts. If no text type is given, a minimal generic system prompt is used which can be refined according to the user's needs.\n\n``` sh\n$ mix magma.text.new \"Example User Guide\" UserGuide\n```\n\n#","ref":"Mix.Tasks.Magma.Text.New.html"},{"type":"task","title":"Command line options - mix magma.text.new","doc":"-   `--force` - When set, this option allows the task to overwrite existing documents without asking for permission first.","ref":"Mix.Tasks.Magma.Text.New.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Magma.Text.New.run/1","doc":"","ref":"Mix.Tasks.Magma.Text.New.html#run/1"},{"type":"task","title":"mix magma.text.type.new","doc":"A Mix task for generating a new text type.\n\nThis task is used to create a configuration document for a new text type. Users must specify a valid Elixir module name as the text type name as the first argument. Optionally, a human-readable label can be provided.\n\n#","ref":"Mix.Tasks.Magma.Text.Type.New.html"},{"type":"task","title":"Example - mix magma.text.type.new","doc":"To create a new text type with just the name:\n\n``` shell\n$ mix magma.text.type.new MyTextType\n```\n\nTo create a new text type with a name and a label:\n\n``` shell\n$ mix magma.text.type.new MyTextType \"My Custom Text Type\"\n```\n\n#","ref":"Mix.Tasks.Magma.Text.Type.New.html#module-example"},{"type":"task","title":"Command line options - mix magma.text.type.new","doc":"-   `--force` - Overwrites any existing text type configuration without confirmation (default: false)","ref":"Mix.Tasks.Magma.Text.Type.New.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Magma.Text.Type.New.run/1","doc":"","ref":"Mix.Tasks.Magma.Text.Type.New.html#run/1"},{"type":"task","title":"mix magma.vault.init","doc":"A Mix task for initializing the Magma vault directory.\n\nThis task is responsible for creating a new Magma vault for your project. The vault is initialized with a given project name, and optionally with a BaseVault which is an Obsidian vault preconfigured with Obsidian themes and plugins. If no BaseVault is specified, the default BaseVault is used. The vault is stored by default under the `docs.magma` directory within your project, this can be configured in the `config.exs` file.\n\nDuring initialization, the task creates a concept document and artefact prompt for the project, concept documents and artefact prompts for all public modules of the project and for their moduledocs via a code sync, and a custom prompt template.\n\n#","ref":"Mix.Tasks.Magma.Vault.Init.html"},{"type":"task","title":"Example - mix magma.vault.init","doc":"``` sh\n$ mix magma.vault.init \"Your project name\"\n```\n\n#","ref":"Mix.Tasks.Magma.Vault.Init.html#module-example"},{"type":"task","title":"Configuration - mix magma.vault.init","doc":"The location where the Magma vault is stored can be changed by setting the `dir` application key in the `magma` app in `config.exs`.\n\n``` elixir\nconfig :magma,  \n  dir: \"your_magma_vault/\"\n```\n\nA set of tags to be added on all generated documents can be configured with the `default_tags` application key. This can be useful for separating Magma documents from other documents in your vault. e.g. to filter them.\n\n``` elixir\nconfig :magma,  \n  default_tags: [\"magma-vault\"]\n```\n\n#","ref":"Mix.Tasks.Magma.Vault.Init.html#module-configuration"},{"type":"task","title":"Command line options - mix magma.vault.init","doc":"-   `--force`: If specified, forces the initialization even if a Magma vault already exists.\n-   `--base-vault`: The name of a BaseVault to be used for initialization. If not specified, the default BaseVault is used.\n-   `--base-vault-path`: The local path to a self-defined BaseVault to be used for initialization.\n-   `--no-code-sync`: If specified, the code sync that creates concept documents and artefact prompts for all public modules of the project is skipped.","ref":"Mix.Tasks.Magma.Vault.Init.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Magma.Vault.Init.run/1","doc":"","ref":"Mix.Tasks.Magma.Vault.Init.html#run/1"},{"type":"task","title":"mix magma.vault.migrate","doc":"Migrates the Magma vault to a newer Magma version","ref":"Mix.Tasks.Magma.Vault.Migrate.html"},{"type":"function","title":"Mix.Tasks.Magma.Vault.Migrate.run/1","doc":"","ref":"Mix.Tasks.Magma.Vault.Migrate.html#run/1"},{"type":"task","title":"mix magma.vault.sync.code","doc":"A Mix task for syncing the module documents in the Magma vault with the ones in the codebase.\n\nThis Mix task is used to create corresponding Magma documents for all public and non-ignored modules in the codebase. This is particularly useful when new modules have been added after the initial vault creation. The task can be run from the command line using `mix magma.vault.sync.code`.\n\nA module is considered ignored and will be skipped in the sync process:\n\n-   If it has a `# Magma pragma: ignore` comment at the beginning of its source code, or\n-   If it is marked as hidden (e.g. with `@moduledoc false`) and does not have a `# Magma pragma: include` comment at the beginning of its source code.\n\nFor each non-ignored module, the following Magma documents are created (unless they already exist):\n\n-   `Magma.Concept`\n-   `Magma.Artefact.Prompt`s for all `Magma.Artefact`s of modules, e.g. a prompt for `Magma.Artefacts.ModuleDoc`\n\n#","ref":"Mix.Tasks.Magma.Vault.Sync.Code.html"},{"type":"task","title":"Configuration - mix magma.vault.sync.code","doc":"A set of tags to be added on all generated documents can be configured with the `default_tags` application key. This can be useful for separating Magma documents from other documents in your vault. e.g. to filter them.\n\n``` elixir\nconfig :magma,  \n  default_tags: [\"magma-vault\"]\n```\n\n#","ref":"Mix.Tasks.Magma.Vault.Sync.Code.html#module-configuration"},{"type":"task","title":"Command line options - mix magma.vault.sync.code","doc":"-   `--force` - Forces the overwrite of all existing documents without user confirmation\n-   `--all` - Includes modules for already existing documents in the synchronization process","ref":"Mix.Tasks.Magma.Vault.Sync.Code.html#module-command-line-options"},{"type":"function","title":"Mix.Tasks.Magma.Vault.Sync.Code.run/1","doc":"","ref":"Mix.Tasks.Magma.Vault.Sync.Code.html#run/1"},{"type":"module","title":"Magma.Vault","doc":"A specialized Obsidian vault with directories for the Magma-specific documents.\n\nThe `Magma.Vault` module serves as a representation and utility module for a Magma vault - a specialized Obsidian vault that resides in an Elixir project. This vault is more than just a collection of Markdown documents; it houses Magma documents, which are special kinds of Markdown documents with specific paths and purposes. The vault itself can be stored by default in the `docs.magma/` directory of an Elixir project but can be reconfigured as needed (see `path/0`).\n\nMain functions of this module include:\n\n- Retrieving paths within the vault, like the base path, template paths, concept paths, etc.\n- Creating and initializing a new vault (`create/3`).\n- Synchronizing the vault with the project's codebase (`sync/1`).\n- Indexing documents by name (`index/1`).\n- Fetching details of documents, such as their path (`document_path/1`) or type (`document_type/1`) .","ref":"Magma.Vault.html"},{"type":"function","title":"Magma.Vault.artefact_generation_path/0","doc":"Returns the Vault path of the directory for `Magma.Artefact.Prompt` documents.","ref":"Magma.Vault.html#artefact_generation_path/0"},{"type":"function","title":"Magma.Vault.artefact_generation_path/1","doc":"Constructs a complete path for `Magma.Artefact.Prompt` documents by joining the specified `segments` to the `artefact_generation_path/0`.","ref":"Magma.Vault.html#artefact_generation_path/1"},{"type":"function","title":"Magma.Vault.artefact_version_path/0","doc":"Returns the Vault path of the directory for `Magma.Artefact.Version` documents.","ref":"Magma.Vault.html#artefact_version_path/0"},{"type":"function","title":"Magma.Vault.artefact_version_path/1","doc":"Constructs a complete path for `Magma.Artefact.Version` documents by joining the specified `segments` to the `artefact_generation_path/0`.","ref":"Magma.Vault.html#artefact_version_path/1"},{"type":"function","title":"Magma.Vault.concept_path/0","doc":"Returns the Vault path of the directory for `Magma.Concept` documents.","ref":"Magma.Vault.html#concept_path/0"},{"type":"function","title":"Magma.Vault.concept_path/1","doc":"Constructs a complete path for `Magma.Concept` documents by joining the specified `segments` to the `concept_path/0`.","ref":"Magma.Vault.html#concept_path/1"},{"type":"function","title":"Magma.Vault.create/3","doc":"Creates and initializes a new vault.\n\nThe `project_name` is a mandatory parameter.\nThe `base_vault` specifies the `Magma.Vault.BaseVault` to be used for\ninitializing the new Magma vault. It can be specified with any of arguments\naccepted by `Magma.Vault.BaseVault.path/1`.\n\nAvailable `opts`:\n\n- `:force` (default: `false`): allow to force creation even if a vault already exists\n- `:code_sync` (default: `true`): perform an initial code `sync/1`\n\n\nReturns `:ok` if the vault is successfully created or an error tuple if\nthere's an error during the vault creation process.","ref":"Magma.Vault.html#create/3"},{"type":"function","title":"Magma.Vault.custom_prompt_template_path/0","doc":"Returns the Vault path for the custom prompt template.","ref":"Magma.Vault.html#custom_prompt_template_path/0"},{"type":"function","title":"Magma.Vault.document_path/1","doc":"Return the path of an existing document.\n\nWhen given a path it checks if there actually exists a document at this path.\nWhen given a document name (without file extension) it trys to fetch the path\nfrom the index.\n\nReturns `nil`, if no file exists at the given path or no document with the\ngiven name is indexed.","ref":"Magma.Vault.html#document_path/1"},{"type":"function","title":"Magma.Vault.document_type/1","doc":"Determines the type of the document with the given `name_or_path`.\n\nThe type is determined by the `magma_type` property within the document's\nYAML front matter.","ref":"Magma.Vault.html#document_type/1"},{"type":"function","title":"Magma.Vault.index/1","doc":"Indexes the provided document by its name.\n\nThis function indexes a given `Magma.Document` to enable fast access to it by\nits name in the `document_path/1` function or the `load/1` functions of\n`Magma.Document`s.","ref":"Magma.Vault.html#index/1"},{"type":"function","title":"Magma.Vault.path/0","doc":"Returns the application configured path to the vault.\n\nUnless specified otherwise, the path is the `docs.magma` directory\ninside the project directory.\n\nIt can be changed with in your `config.exs` file like this:\n\n    config :magma,\n      dir: \"custom_dir\"\n\nNote, that this configuration should be environment-independent.\nUnless you're working on Magma itself, you don't want a test-specific vault,\nsince the vault collects knowledge about your code in its entirety.","ref":"Magma.Vault.html#path/0"},{"type":"function","title":"Magma.Vault.path/1","doc":"Constructs a complete path by joining the specified `segments` to the root vault `path/0`.\n\nMost of the time one of the more document type-specific functions is more suitable.\n\n#","ref":"Magma.Vault.html#path/1"},{"type":"function","title":"Example - Magma.Vault.path/1","doc":"Magma.Vault.path(\"directory\")\n    # returns: \"/path/to/project/docs.magma/directory\"\n\n    Magma.Vault.path([\"some\", \"directory\"])\n    # returns: \"/path/to/project/docs.magma/some/directory\"","ref":"Magma.Vault.html#path/1-example"},{"type":"function","title":"Magma.Vault.sync/1","doc":"Synchronizes the `Magma.Matter.Module` related documents with the latest state of the codebase.\n\nAll modules in the code base are determined and for each one the following\n`Magma.Document`s created (unless they exist already or the `:all` option is set):\n\n- a `Magma.Concept`\n- `Magma.Artefact.Prompt`s for all `Magma.Artefact`s for `Magma.Matter.Module`\n  (as specified by `Magma.Matter.Module.artefacts/0`)\n\nAvailable options:\n\n- `:all` (default: `false`) - when set to `true` also syncs modules for\n  already existing documents\n- `force` (default: `false`) - when set to `true` overwrites all existing\n  documents without asking the user","ref":"Magma.Vault.html#sync/1"},{"type":"function","title":"Magma.Vault.template_path/0","doc":"Returns the Vault path of the directory for templates.","ref":"Magma.Vault.html#template_path/0"},{"type":"function","title":"Magma.Vault.template_path/1","doc":"Constructs a complete template path by joining the specified `segments` to the `template_path/0`.\n\n#","ref":"Magma.Vault.html#template_path/1"},{"type":"function","title":"Example - Magma.Vault.template_path/1","doc":"Magma.Vault.template_path(\"some_template.md\")\n    # returns: \"/path/to/project/docs.magma/templates/some_template.md\"","ref":"Magma.Vault.html#template_path/1-example"},{"type":"module","title":"Magma.Vault.BaseVault","doc":"Provides utilities to interact with predefined and custom base vaults.\n\nA base vault is a preconfigured Obsidian vault that serves as a template when initializing a new Magma vault.\n\n#","ref":"Magma.Vault.BaseVault.html"},{"type":"module","title":"Creating a new base vault - Magma.Vault.BaseVault","doc":"If you are looking to create a new base vault (either a local one or as a contribution to the Magma project), ensure you include the required plugins from the default base vault:\n\n-   [Buttons](https://github.com/shabegom/buttons)\n-   [Shell commands](https://github.com/Taitava/obsidian-shellcommands)\n-   [QuickAdd](https://github.com/chhoumann/quickadd)\n-   [Dataview](https://github.com/blacksmithgu/obsidian-dataview)\n\nAlso, it's vital to copy the configurations of the Shell Commands and QuickAdd plugins, as they include the integration with the respective Magma mix tasks.","ref":"Magma.Vault.BaseVault.html#module-creating-a-new-base-vault"},{"type":"function","title":"Magma.Vault.BaseVault.path/1","doc":"Returns the path to a base vault.\n\nEither the atom name of one of the predefined base vault in the `priv/base_vault`\ndirectory of Magma can be used or the path to a custom local base vault.\nIf no base vault is given the default base vault is used.\n\n    # Get path for the default base vault\n    Magma.Vault.BaseVault.path()\n\n    # Get path for a predefined base vault\n    Magma.Vault.BaseVault.path(:custom_theme)\n\n    # Get path for a custom base vault\n    Magma.Vault.BaseVault.path(\"/path/to/custom/base/vault\")","ref":"Magma.Vault.BaseVault.html#path/1"},{"type":"function","title":"Magma.Vault.BaseVault.path!/1","doc":"Returns the path to a base vault and raises an error when the given base vault does not exist.\n\nAccepts the same arguments as `path/1`.","ref":"Magma.Vault.BaseVault.html#path!/1"},{"type":"type","title":"Magma.Vault.BaseVault.theme/0","doc":"","ref":"Magma.Vault.BaseVault.html#t:theme/0"},{"type":"module","title":"Magma.Vault.Migration","doc":"Migration of Magma vaults to be compatible with newer versions.\n\nThis module implements the general migration logic for `Magma.Vault`s\ncreated with older versions of Magma, so they can be updated to work with\na newer version. It provides functionality to check the vault's version\nagainst the required version and apply any necessary migrations, implemented\ndedicated modules for specific versions. This process is crucial for\nmaintaining consistency and functionality as the Magma project evolves.","ref":"Magma.Vault.Migration.html"},{"type":"function","title":"Magma.Vault.Migration.magma_version_requirement/0","doc":"","ref":"Magma.Vault.Migration.html#magma_version_requirement/0"},{"type":"function","title":"Magma.Vault.Migration.migrate/0","doc":"Applies all necessary migrations to update the vault for a newer version of Magma.","ref":"Magma.Vault.Migration.html#migrate/0"},{"type":"module","title":"Magma.Vault.Version","doc":"Manages the versioning of the Magma vault.\n\nThe `Magma.Vault.Version` module provides functionality to handle the\nversioning of the `Magma.Vault` by managing a version file within the vault.\nThis is crucial for ensuring compatibility between the vault and the\nversion of Magma being used, and for performing migrations when upgrading\nto a new version of Magma.","ref":"Magma.Vault.Version.html"},{"type":"function","title":"Magma.Vault.Version.file/0","doc":"Returns the path to the version file in the Magma vault.","ref":"Magma.Vault.Version.html#file/0"},{"type":"function","title":"Magma.Vault.Version.load/0","doc":"Loads and returns the Magma vault version from the version file.\n\nThis function reads the version information from the `.VERSION` file within\nthe Magma vault. If the version file exists, it parses the version string\ninto a `Version` struct. If the file does not exist, it defaults to version\n\"0.1.0\".","ref":"Magma.Vault.Version.html#load/0"},{"type":"function","title":"Magma.Vault.Version.save/1","doc":"Saves the given version to the version file in the Magma vault.","ref":"Magma.Vault.Version.html#save/1"},{"type":"module","title":"Magma.Artefact.Prompt","doc":"","ref":"Magma.Artefact.Prompt.html"},{"type":"function","title":"Magma.Artefact.Prompt.create/3","doc":"","ref":"Magma.Artefact.Prompt.html#create/3"},{"type":"function","title":"Magma.Artefact.Prompt.from!/1","doc":"Fetches a document from a related document and immediately loads it with `load!/1`","ref":"Magma.Artefact.Prompt.html#from!/1"},{"type":"function","title":"Magma.Artefact.Prompt.load/1","doc":"Loads a document from the given `path` or `document`.\n\nIf the loaded document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Artefact.Prompt.html#load/1"},{"type":"function","title":"Magma.Artefact.Prompt.load!/1","doc":"Loads a document from the given `path` or `document` and raises an exception in error cases.","ref":"Magma.Artefact.Prompt.html#load!/1"},{"type":"function","title":"Magma.Artefact.Prompt.load_linked/1","doc":"Loads a document from the given Obsidian `link`, i.e. a string of the form `\"[[name]]\"`.\n\nIf the referenced document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Artefact.Prompt.html#load_linked/1"},{"type":"function","title":"Magma.Artefact.Prompt.new/2","doc":"","ref":"Magma.Artefact.Prompt.html#new/2"},{"type":"function","title":"Magma.Artefact.Prompt.new!/2","doc":"","ref":"Magma.Artefact.Prompt.html#new!/2"},{"type":"function","title":"Magma.Artefact.Prompt.render/1","doc":"","ref":"Magma.Artefact.Prompt.html#render/1"},{"type":"type","title":"Magma.Artefact.Prompt.t/0","doc":"","ref":"Magma.Artefact.Prompt.html#t:t/0"},{"type":"module","title":"Magma.Artefact.Version","doc":"","ref":"Magma.Artefact.Version.html"},{"type":"function","title":"Magma.Artefact.Version.create/3","doc":"","ref":"Magma.Artefact.Version.html#create/3"},{"type":"function","title":"Magma.Artefact.Version.from!/1","doc":"Fetches a document from a related document and immediately loads it with `load!/1`","ref":"Magma.Artefact.Version.html#from!/1"},{"type":"function","title":"Magma.Artefact.Version.load/1","doc":"Loads a document from the given `path` or `document`.\n\nIf the loaded document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Artefact.Version.html#load/1"},{"type":"function","title":"Magma.Artefact.Version.load!/1","doc":"Loads a document from the given `path` or `document` and raises an exception in error cases.","ref":"Magma.Artefact.Version.html#load!/1"},{"type":"function","title":"Magma.Artefact.Version.load_linked/1","doc":"Loads a document from the given Obsidian `link`, i.e. a string of the form `\"[[name]]\"`.\n\nIf the referenced document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Artefact.Version.html#load_linked/1"},{"type":"function","title":"Magma.Artefact.Version.new/2","doc":"","ref":"Magma.Artefact.Version.html#new/2"},{"type":"function","title":"Magma.Artefact.Version.new!/2","doc":"","ref":"Magma.Artefact.Version.html#new!/2"},{"type":"type","title":"Magma.Artefact.Version.t/0","doc":"","ref":"Magma.Artefact.Version.html#t:t/0"},{"type":"module","title":"Magma.Concept","doc":"The basic Magma document type used for generating concrete artefacts.\n\nIt contains all user-contributed content necessary for artefact generation,\nsuch as descriptions of the subject matter, background knowledge, and task\ndescriptions for various artefacts.\n\nThis module provides functions for creating, loading and updating these\nconcept documents, as well as functions for accessing specific sections\nof the document such as the description and context knowledge sections.\nIt also allows for the creation of prompts based on a concept.","ref":"Magma.Concept.html"},{"type":"function","title":"Magma.Concept.context_knowledge_section/1","doc":"","ref":"Magma.Concept.html#context_knowledge_section/1"},{"type":"function","title":"Magma.Concept.context_knowledge_section_title/0","doc":"","ref":"Magma.Concept.html#context_knowledge_section_title/0"},{"type":"function","title":"Magma.Concept.create/3","doc":"Creates a new concept document for a given subject matter in the `Magma.Vault`.","ref":"Magma.Concept.html#create/3"},{"type":"function","title":"Magma.Concept.create!/3","doc":"Creates a new concept document for a given subject matter in the `Magma.Vault`.\n\nFails in error cases.","ref":"Magma.Concept.html#create!/3"},{"type":"function","title":"Magma.Concept.create_prompts/2","doc":"","ref":"Magma.Concept.html#create_prompts/2"},{"type":"function","title":"Magma.Concept.description_section/1","doc":"","ref":"Magma.Concept.html#description_section/1"},{"type":"function","title":"Magma.Concept.description_section_title/0","doc":"","ref":"Magma.Concept.html#description_section_title/0"},{"type":"function","title":"Magma.Concept.fetch/2","doc":"","ref":"Magma.Concept.html#fetch/2"},{"type":"function","title":"Magma.Concept.from!/1","doc":"Fetches a document from a related document and immediately loads it with `load!/1`","ref":"Magma.Concept.html#from!/1"},{"type":"function","title":"Magma.Concept.load/1","doc":"Loads a document from the given `path` or `document`.\n\nIf the loaded document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Concept.html#load/1"},{"type":"function","title":"Magma.Concept.load!/1","doc":"Loads a document from the given `path` or `document` and raises an exception in error cases.","ref":"Magma.Concept.html#load!/1"},{"type":"function","title":"Magma.Concept.load_linked/1","doc":"Loads a document from the given Obsidian `link`, i.e. a string of the form `\"[[name]]\"`.\n\nIf the referenced document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Concept.html#load_linked/1"},{"type":"function","title":"Magma.Concept.new/2","doc":"Creates a new concept document struct for a given subject matter.\n\nNote, this function doesn't create the document in the `Magma.Vault`.\nUse `create/3` for this purpose.","ref":"Magma.Concept.html#new/2"},{"type":"function","title":"Magma.Concept.new!/2","doc":"","ref":"Magma.Concept.html#new!/2"},{"type":"function","title":"Magma.Concept.update_content_from_ast/1","doc":"","ref":"Magma.Concept.html#update_content_from_ast/1"},{"type":"type","title":"Magma.Concept.t/0","doc":"","ref":"Magma.Concept.html#t:t/0"},{"type":"behaviour","title":"Magma.Document","doc":"A behavior for the different kinds of document types in Magma.\n\nBesides the callback definition, it provides shared fields and logic between\nall document types. Each document type defines additional fields for its\nspecific tasks and a path scheme that determines where instances of this type\nare stored.\n\nNote, that in general the content under the YAML frontmatter of a document\nis not further interpreted (except `Magma.Concept`).\n`Magma.DocumentStruct` allows to get the AST of a Markdown document.","ref":"Magma.Document.html"},{"type":"callback","title":"Magma.Document.build_path/1","doc":"Builds the path of a new document during its creation.\n\nThe function will receive a struct created with the respective `new` function, which should\nhave initialized all parts required for this path building step.","ref":"Magma.Document.html#c:build_path/1"},{"type":"function","title":"Magma.Document.fields/0","doc":"","ref":"Magma.Document.html#fields/0"},{"type":"callback","title":"Magma.Document.from/1","doc":"Fetches a document from a related document.\n\nFor example, `Concept.from(prompt)` will return the `Magma.Concept` document from the given `prompt`.\n\nImplementation should implement clauses for all document types for which it is possible.\n\nFor `Magma.Artefact`-specific documents from a `Magma.Concept`, the concept must be given in a\n`{Concept.t(), Artefact.t()}` tuple.","ref":"Magma.Document.html#c:from/1"},{"type":"callback","title":"Magma.Document.load_document/1","doc":"Document type specific logic when loading a document.\n\nUsually the document type specific fields of the YAML front matter of the document\nare extracted and interpreted here.","ref":"Magma.Document.html#c:load_document/1"},{"type":"function","title":"Magma.Document.recreate/1","doc":"Creates the file for document, overwriting the existing one.\n\nThis function is used by the `Mix.Tasks.Magma.Prompt.Update` Mix task.","ref":"Magma.Document.html#recreate/1"},{"type":"function","title":"Magma.Document.recreate!/1","doc":"Creates the file for document, overwriting the existing one.\n\nThis function is used by the `Mix.Tasks.Magma.Prompt.Update` Mix task.","ref":"Magma.Document.html#recreate!/1"},{"type":"callback","title":"Magma.Document.render_front_matter/1","doc":"Renders the document type specific fields as YAML front matter lines.","ref":"Magma.Document.html#c:render_front_matter/1"},{"type":"function","title":"Magma.Document.save/2","doc":"Saves the changes on a document.","ref":"Magma.Document.html#save/2"},{"type":"callback","title":"Magma.Document.title/1","doc":"The title of the document used in the initial top-level header of a document.","ref":"Magma.Document.html#c:title/1"},{"type":"function","title":"Magma.Document.type/1","doc":"Returns the document type module for the given string.","ref":"Magma.Document.html#type/1"},{"type":"function","title":"Example - Magma.Document.type/1","doc":"iex> Magma.Document.type(\"Concept\")\n    Magma.Concept\n\n    iex> Magma.Document.type(\"Artefact.Prompt\")\n    Magma.Artefact.Prompt\n\n    iex> Magma.Document.type(\"Config.System\")\n    Magma.Config.System\n\n    iex> Magma.Document.type(\"Vault\")\n    nil\n\n    iex> Magma.Document.type(\"NonExisting\")\n    nil","ref":"Magma.Document.html#type/1-example"},{"type":"function","title":"Magma.Document.type?/1","doc":"Returns if the given module is a document type module.","ref":"Magma.Document.html#type?/1"},{"type":"function","title":"Magma.Document.type_name/1","doc":"Returns the document type name for the given document.","ref":"Magma.Document.html#type_name/1"},{"type":"function","title":"Example - Magma.Document.type_name/1","doc":"iex> Magma.Document.type_name(Magma.Concept)\n    \"Concept\"\n\n    iex> Magma.Document.type_name(Magma.Prompt)\n    \"Prompt\"\n\n    iex> Magma.Document.type_name(Magma.Artefact.Prompt)\n    \"Artefact.Prompt\"\n\n    iex> Magma.Document.type_name(Magma.PromptResult)\n    \"PromptResult\"\n\n    iex> Magma.Document.type_name(Magma.Artefact.Version)\n    \"Artefact.Version\"\n\n    iex> Magma.Document.type_name(Magma.Text.Preview)\n    \"Text.Preview\"\n\n    iex> Magma.Document.type_name(Magma.Vault)\n    ** (RuntimeError) Invalid Magma.Document type: Magma.Vault\n\n    iex> Magma.Document.type_name(NonExisting)\n    ** (RuntimeError) Invalid Magma.Document type: NonExisting","ref":"Magma.Document.html#type_name/1-example"},{"type":"type","title":"Magma.Document.t/0","doc":"","ref":"Magma.Document.html#t:t/0"},{"type":"type","title":"Magma.Document.type/0","doc":"","ref":"Magma.Document.html#t:type/0"},{"type":"module","title":"Magma.Prompt","doc":"","ref":"Magma.Prompt.html"},{"type":"function","title":"Magma.Prompt.create/3","doc":"","ref":"Magma.Prompt.html#create/3"},{"type":"function","title":"Magma.Prompt.from!/1","doc":"Fetches a document from a related document and immediately loads it with `load!/1`","ref":"Magma.Prompt.html#from!/1"},{"type":"function","title":"Magma.Prompt.load/1","doc":"Loads a document from the given `path` or `document`.\n\nIf the loaded document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Prompt.html#load/1"},{"type":"function","title":"Magma.Prompt.load!/1","doc":"Loads a document from the given `path` or `document` and raises an exception in error cases.","ref":"Magma.Prompt.html#load!/1"},{"type":"function","title":"Magma.Prompt.load_linked/1","doc":"Loads a document from the given Obsidian `link`, i.e. a string of the form `\"[[name]]\"`.\n\nIf the referenced document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Prompt.html#load_linked/1"},{"type":"function","title":"Magma.Prompt.new/2","doc":"","ref":"Magma.Prompt.html#new/2"},{"type":"function","title":"Magma.Prompt.new!/2","doc":"","ref":"Magma.Prompt.html#new!/2"},{"type":"function","title":"Magma.Prompt.path_prefix/0","doc":"","ref":"Magma.Prompt.html#path_prefix/0"},{"type":"function","title":"Magma.Prompt.render/1","doc":"","ref":"Magma.Prompt.html#render/1"},{"type":"type","title":"Magma.Prompt.t/0","doc":"","ref":"Magma.Prompt.html#t:t/0"},{"type":"module","title":"Magma.PromptResult","doc":"","ref":"Magma.PromptResult.html"},{"type":"function","title":"Magma.PromptResult.build_name/1","doc":"","ref":"Magma.PromptResult.html#build_name/1"},{"type":"function","title":"Magma.PromptResult.controls/1","doc":"","ref":"Magma.PromptResult.html#controls/1"},{"type":"function","title":"Magma.PromptResult.create/3","doc":"","ref":"Magma.PromptResult.html#create/3"},{"type":"function","title":"Magma.PromptResult.dir/0","doc":"","ref":"Magma.PromptResult.html#dir/0"},{"type":"function","title":"Magma.PromptResult.from!/1","doc":"Fetches a document from a related document and immediately loads it with `load!/1`","ref":"Magma.PromptResult.html#from!/1"},{"type":"function","title":"Magma.PromptResult.load/1","doc":"Loads a document from the given `path` or `document`.\n\nIf the loaded document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.PromptResult.html#load/1"},{"type":"function","title":"Magma.PromptResult.load!/1","doc":"Loads a document from the given `path` or `document` and raises an exception in error cases.","ref":"Magma.PromptResult.html#load!/1"},{"type":"function","title":"Magma.PromptResult.load_linked/1","doc":"Loads a document from the given Obsidian `link`, i.e. a string of the form `\"[[name]]\"`.\n\nIf the referenced document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.PromptResult.html#load_linked/1"},{"type":"function","title":"Magma.PromptResult.new/2","doc":"","ref":"Magma.PromptResult.html#new/2"},{"type":"function","title":"Magma.PromptResult.new!/2","doc":"","ref":"Magma.PromptResult.html#new!/2"},{"type":"type","title":"Magma.PromptResult.t/0","doc":"","ref":"Magma.PromptResult.html#t:t/0"},{"type":"module","title":"Magma.DocumentStruct","doc":"Provides an abstract representation of a Markdown document structured based on the Pandoc AST.\n\nThe `Magma.DocumentStruct` module provides an Elixir struct for representing\nthe contents of a Markdown document as an Abstract Syntax Tree (AST) based\non the Pandoc AST. The struct is designed to access the individual sections\nincluding their subsections and facilitate the transclusion resolution feature,\nwhich is essential for the prompt generation in Magma.\n\nThe `Magma.DocumentStruct` struct consists of a prologue, which is the\nheader-less text before the first section, and all sections of level 1\n(which in turn consist of sections of level 2 and so on).\nThe core functionalities related to sections are implemented in the\n`Magma.DocumentStruct.Section` module. The `Magma.DocumentStruct` acts as a\nwrapper around this recursive section structure and delegates most of its\nfunctions to the said module.","ref":"Magma.DocumentStruct.html"},{"type":"function","title":"Magma.DocumentStruct.fetch/2","doc":"Fetches the section with the given `title` and returns it in an ok tuple.\n\nIf no section with `title` exists, it returns `:error`.\n\nThis implements `Access.fetch/2` function, so that the `document_struct[title]`\nsyntax and the `Kernel` macros for accessing nested data structures like\n`get_in/2` are supported.\n\nThis function only searches sections directly under the given section.\nFor a recursive search, use `section_by_title/2`.","ref":"Magma.DocumentStruct.html#fetch/2"},{"type":"function","title":"Magma.DocumentStruct.main_section/1","doc":"Returns the first section.\n\nAssuming that the first section with header level 1 is the main section.","ref":"Magma.DocumentStruct.html#main_section/1"},{"type":"function","title":"Magma.DocumentStruct.parse/1","doc":"Parses the given content into a `Magma.DocumentStruct`.","ref":"Magma.DocumentStruct.html#parse/1"},{"type":"function","title":"Magma.DocumentStruct.remove_comments/1","doc":"Removes all comment blocks from the given `document_struct`.\n\nSee `Magma.DocumentStruct.Section.remove_comments/1` which does the same\non a section level.","ref":"Magma.DocumentStruct.html#remove_comments/1"},{"type":"function","title":"Magma.DocumentStruct.resolve_transclusions/1","doc":"Processes and resolves transclusions within the given `document_struct`.\n\nSee `Magma.DocumentStruct.Section.resolve_transclusions/1` which does the same\non a section level.","ref":"Magma.DocumentStruct.html#resolve_transclusions/1"},{"type":"function","title":"Magma.DocumentStruct.section_by_title/2","doc":"Fetches the first section with the given `title`.\n\nUnlike `fetch/2`, this function performs a recursive search throughout the\ndocument to find the desired section.","ref":"Magma.DocumentStruct.html#section_by_title/2"},{"type":"function","title":"Magma.DocumentStruct.set_level/2","doc":"Sets the header level for all sections within the document.\n\nSee `Magma.DocumentStruct.Section.set_level/2` which does the same\non a section level.","ref":"Magma.DocumentStruct.html#set_level/2"},{"type":"function","title":"Magma.DocumentStruct.title/1","doc":"Extracts and returns the title of the `main_section/1`.","ref":"Magma.DocumentStruct.html#title/1"},{"type":"function","title":"Magma.DocumentStruct.to_markdown/1","doc":"Converts the given `document_struct` back into a Markdown string.","ref":"Magma.DocumentStruct.html#to_markdown/1"},{"type":"type","title":"Magma.DocumentStruct.compatible/0","doc":"","ref":"Magma.DocumentStruct.html#t:compatible/0"},{"type":"type","title":"Magma.DocumentStruct.t/0","doc":"","ref":"Magma.DocumentStruct.html#t:t/0"},{"type":"module","title":"Magma.DocumentStruct.Section","doc":"Recursive structure for the nested sections of a `Magma.DocumentStruct`.","ref":"Magma.DocumentStruct.Section.html"},{"type":"function","title":"Magma.DocumentStruct.Section.empty?/1","doc":"Checks if the given `section` is empty, i.e. it has no `content` and nested `sections`.","ref":"Magma.DocumentStruct.Section.html#empty?/1"},{"type":"function","title":"Magma.DocumentStruct.Section.empty_content?/1","doc":"Checks if the given `section` consists solely of subsection headers.","ref":"Magma.DocumentStruct.Section.html#empty_content?/1"},{"type":"function","title":"Magma.DocumentStruct.Section.fetch/2","doc":"Fetches the section with the given `title` and returns it in an ok tuple.\n\nIf no section with `section` exists, it returns `:error`.\n\nThis implements `Access.fetch/2` function, so that the `section[title]`\nsyntax and the `Kernel` macros for accessing nested data structures like\n`get_in/2` are supported.\n\nThis function only searches sections directly under the given section.\nFor a recursive search, use `section_by_title/2`.","ref":"Magma.DocumentStruct.Section.html#fetch/2"},{"type":"function","title":"Magma.DocumentStruct.Section.new/3","doc":"Creates a new section.","ref":"Magma.DocumentStruct.Section.html#new/3"},{"type":"function","title":"Magma.DocumentStruct.Section.preserve_eex_tags/1","doc":"","ref":"Magma.DocumentStruct.Section.html#preserve_eex_tags/1"},{"type":"function","title":"Magma.DocumentStruct.Section.remove_comments/1","doc":"Removes all comment blocks from the given `section_or_content`.\n\nThis function cleans up the document by removing all comment blocks\n(`<!-- comment -->`).","ref":"Magma.DocumentStruct.Section.html#remove_comments/1"},{"type":"function","title":"Magma.DocumentStruct.Section.resolve_links/2","doc":"Resolves internal links in the provided `section` by replacing them with their content or display text.\n\nThe style of the resolved links can be specified with the `:style` option,\nwhich accepts the following values:\n\n- `:plain` (default) - no styling\n- `:emph` - italic\n- `:strong` - bold\n- `:underline` - underlined\n- a function accepting the children of the link AST and returning the\n  replacement AST node","ref":"Magma.DocumentStruct.Section.html#resolve_links/2"},{"type":"function","title":"Magma.DocumentStruct.Section.resolve_transclusions/1","doc":"Processes and resolves transclusions within the given `section`.\n\nTransclusion resolution in Magma is a procedure where an Obsidian transclusion,\nsuch as `![[Some document]]`, is replaced with its actual content.\nThis mechanism forms the foundation for constructing LLM prompts in Magma.\nThe content from the referenced document or section undergoes several processing\nsteps before its insertion:\n\n- Comments (`<!-- comment -->`) are removed.\n- Internal links are replaced with the target as plain text.\n- Transclusions within the transcluded content itself are resolved recursively\n  (unless it would result in an infinite recursion).\n- If the transcluded content (after removing the comments), consists\n  exclusively of a heading with no content below it, the transclusion is\n  resolved with the empty string.\n- The level of the transcluded sections is adjusted according to the current\n  level at the point of the transclusion.\n\nDifferent types of transclusions are resolved in slightly varied ways,\nparticularly regarding the leading header of the transcluded content:\n\n- _Inline transclusions_: Exclude the leading header.\n- _Custom header transclusions_: Replace the leading header.\n- _Empty header transclusions_: Retain the leading header.","ref":"Magma.DocumentStruct.Section.html#resolve_transclusions/1"},{"type":"function","title":"Magma.DocumentStruct.Section.section_by_title/2","doc":"Fetches the first section with the given `title`.\n\nOther than accessing the sections with the `fetch/2`, this searches the\nsections recursively.","ref":"Magma.DocumentStruct.Section.html#section_by_title/2"},{"type":"function","title":"Magma.DocumentStruct.Section.set_header/2","doc":"Sets a new `header` for the given `section`, updating the `:title` and `:level` fields accordingly.","ref":"Magma.DocumentStruct.Section.html#set_header/2"},{"type":"function","title":"Magma.DocumentStruct.Section.set_header/3","doc":"Sets a new header with the given title and level for the given `section`, updating the `:title` and `:level` fields accordingly.","ref":"Magma.DocumentStruct.Section.html#set_header/3"},{"type":"function","title":"Magma.DocumentStruct.Section.set_level/2","doc":"Changes the header level of `section` to the given `level`.\n\nComputes the difference to the current level of `section` and shifts the\nlevel recursively on all subsections using `shift_level/2`.","ref":"Magma.DocumentStruct.Section.html#set_level/2"},{"type":"function","title":"Magma.DocumentStruct.Section.shift_level/2","doc":"Shifts the header level of `section` by the given `shift_level`.\n\nAll subsections are shifted recursively.","ref":"Magma.DocumentStruct.Section.html#shift_level/2"},{"type":"function","title":"Magma.DocumentStruct.Section.to_markdown/2","doc":"Converts a `Magma.DocumentStruct.Section` into a Markdown string.","ref":"Magma.DocumentStruct.Section.html#to_markdown/2"},{"type":"type","title":"Magma.DocumentStruct.Section.t/0","doc":"","ref":"Magma.DocumentStruct.Section.html#t:t/0"},{"type":"behaviour","title":"Magma.Matter","doc":"Behaviour for types of matter that can be subject of a `Magma.Concept` and the `Magma.Artefact`s generated from these concepts.\n\nThis module defines a set of callbacks that each matter type must implement.\nThese callbacks allow for the specification of various properties and\nbehaviours of the matter type, such as the available artefacts, paths for\ndifferent kinds of documents, texts for different parts of the concept and\nprompt documents, and more.","ref":"Magma.Matter.html"},{"type":"callback","title":"Magma.Matter.artefacts/0","doc":"A callback that returns the list of `Magma.Artefact` types that are available for this matter type.","ref":"Magma.Matter.html#c:artefacts/0"},{"type":"callback","title":"Magma.Matter.concept_name/1","doc":"A callback that returns the name of the `Magma.Concept` document.\n\nNote that this name must unique across all document names in the vault.","ref":"Magma.Matter.html#c:concept_name/1"},{"type":"callback","title":"Magma.Matter.concept_title/1","doc":"A callback that returns the title header text of the `Magma.Concept` document.","ref":"Magma.Matter.html#c:concept_title/1"},{"type":"callback","title":"Magma.Matter.context_knowledge/1","doc":"A callback that allows to specify texts which should be included generally in the \"Context knowledge\" section of the `Magma.Artefact.Prompt` document about this type of matter.","ref":"Magma.Matter.html#c:context_knowledge/1"},{"type":"callback","title":"Magma.Matter.custom_concept_sections/1","doc":"A callback that can be used to define additional sections for the `Magma.Concept` document.","ref":"Magma.Matter.html#c:custom_concept_sections/1"},{"type":"callback","title":"Magma.Matter.default_concept_aliases/1","doc":"A callback that returns a list of Obsidian aliases for the `Magma.Concept` document of this type of matter.","ref":"Magma.Matter.html#c:default_concept_aliases/1"},{"type":"callback","title":"Magma.Matter.default_description/2","doc":"A callback that returns a text for the body of the \"Description\" section in the `Magma.Concept` document.\n\nAs the description is something written by the user, this should return\na comment with a hint of what is expected to be written.","ref":"Magma.Matter.html#c:default_description/2"},{"type":"function","title":"Magma.Matter.extract_from_metadata/3","doc":"Extracts an instance of a matter from the matter-specific fields of the metadata of a `Magma.Concept` document.\n\nThis function first extracts the matter type from the `magma_matter_type` field\nin the YAML frontmatter and then delegates to the `c:extract_from_metadata/3`\nimplementation to process the matter-type specific fields.","ref":"Magma.Matter.html#extract_from_metadata/3"},{"type":"callback","title":"Magma.Matter.extract_from_metadata/3","doc":"A callback that extracts an instance of this matter type from the matter-specific fields of the metadata during deserialization of a `Magma.Concept` document.\n\nAll YAML frontmatter properties are loaded first into the `:custom_metadata`\nmap of a `Magma.Document`. This callback implementation should `Map.pop/2` the\nmatter-specific entries from the given `document_metadata` and return the created\ninstance of this matter type and the consumed metadata in an ok tuple.\n\nCounterpart of `render_front_matter/1`.","ref":"Magma.Matter.html#c:extract_from_metadata/3"},{"type":"function","title":"Magma.Matter.fields/0","doc":"Returns a list of the shared fields of the structs of every type of `Magma.Matter`.\n\n    iex> Magma.Matter.fields()\n    [:name]","ref":"Magma.Matter.html#fields/0"},{"type":"callback","title":"Magma.Matter.prompt_concept_description_title/1","doc":"A callback that returns the section title for the concept description of a type of matter in the `Magma.Artefact.Prompt`.","ref":"Magma.Matter.html#c:prompt_concept_description_title/1"},{"type":"callback","title":"Magma.Matter.prompt_matter_description/1","doc":"A callback that can be used to define a general description of some matter which should be included in the `Magma.Artefact.Prompt`.\n\nThis is used for example to include the code of module, in the case of `Magma.Matter.Module`.","ref":"Magma.Matter.html#c:prompt_matter_description/1"},{"type":"callback","title":"Magma.Matter.relative_base_path/1","doc":"A callback that returns the path segment to be used for different kinds of documents for this type of matter.\n\nThis path segment will be incorporated in the path generator functions\nof the `Magma.Document` types.","ref":"Magma.Matter.html#c:relative_base_path/1"},{"type":"callback","title":"Magma.Matter.relative_concept_path/1","doc":"A callback that returns the path for `Magma.Concept` documents about this type of matter.\n\nThis path is relative to the `Magma.Vault.concept_path/0`","ref":"Magma.Matter.html#c:relative_concept_path/1"},{"type":"callback","title":"Magma.Matter.render_front_matter/1","doc":"A callback that renders the matter-specific fields of this type of matter to YAML frontmatter.\n\nCounterpart of `extract_from_metadata/3`.","ref":"Magma.Matter.html#c:render_front_matter/1"},{"type":"function","title":"Magma.Matter.type/1","doc":"Returns the matter type module for the given string.","ref":"Magma.Matter.html#type/1"},{"type":"function","title":"Example - Magma.Matter.type/1","doc":"iex> Magma.Matter.type(\"Module\")\n    Magma.Matter.Module\n\n    iex> Magma.Matter.type(\"Project\")\n    Magma.Matter.Project\n\n    iex> Magma.Matter.type(\"Vault\")\n    nil\n\n    iex> Magma.Matter.type(\"NonExisting\")\n    nil","ref":"Magma.Matter.html#type/1-example"},{"type":"function","title":"Magma.Matter.type?/1","doc":"Checks if the given `module` is a `Magma.Matter` type module.","ref":"Magma.Matter.html#type?/1"},{"type":"function","title":"Magma.Matter.type_name/2","doc":"Returns the matter type name for the given matter type module.","ref":"Magma.Matter.html#type_name/2"},{"type":"function","title":"Example - Magma.Matter.type_name/2","doc":"iex> Magma.Matter.type_name(Magma.Matter.Module)\n    \"Module\"\n\n    iex> Magma.Matter.type_name(Magma.Matter.Text)\n    \"Text\"\n\n    iex> Magma.Matter.type_name(Magma.Matter.Text.Section)\n    \"Text.Section\"\n\n    iex> Magma.Matter.type_name(Magma.Vault)\n    ** (RuntimeError) Invalid Magma.Matter type: Magma.Vault\n\n    iex> Magma.Matter.type_name(NonExisting)\n    ** (RuntimeError) Invalid Magma.Matter type: NonExisting","ref":"Magma.Matter.html#type_name/2-example"},{"type":"type","title":"Magma.Matter.name/0","doc":"","ref":"Magma.Matter.html#t:name/0"},{"type":"type","title":"Magma.Matter.t/0","doc":"","ref":"Magma.Matter.html#t:t/0"},{"type":"type","title":"Magma.Matter.type/0","doc":"","ref":"Magma.Matter.html#t:type/0"},{"type":"module","title":"Magma.Matter.Module","doc":"`Magma.Matter` type behaviour implementation for Elixir modules.\n\nThe `Magma.Matter.Module` struct is used for generation of `Magma.Artefact`s\nabout Elixir modules. It does not have any additional fields above the\n`Magma.Matter.fields/0` as it retrieves all necessary information via the\nElixir and Erlang reflection API from the module name.","ref":"Magma.Matter.Module.html"},{"type":"function","title":"Magma.Matter.Module.artefacts/0","doc":"Returns the list of `Magma.Artefact` types available for Elixir modules.\n\n    iex> Magma.Matter.Module.artefacts()\n    [Magma.Artefacts.ModuleDoc]","ref":"Magma.Matter.Module.html#artefacts/0"},{"type":"function","title":"Magma.Matter.Module.code/1","doc":"Returns the source code of the module, if it exists.\n\nThe source code is read from the `source_path/1`.","ref":"Magma.Matter.Module.html#code/1"},{"type":"function","title":"Magma.Matter.Module.concept_name/1","doc":"Returns the name of the `Magma.Concept` document for an Elixir module.\n\nIt is the module name as a string.\n\n#","ref":"Magma.Matter.Module.html#concept_name/1"},{"type":"function","title":"Example - Magma.Matter.Module.concept_name/1","doc":"iex> Some.Module\n  ...> |> Magma.Matter.Module.new!()\n  ...> |> Magma.Matter.Module.concept_name()\n  \"Some.Module\"","ref":"Magma.Matter.Module.html#concept_name/1-example"},{"type":"function","title":"Magma.Matter.Module.concept_title/1","doc":"Returns the title header text of the `Magma.Concept` document for an Elixir module.\n\n#","ref":"Magma.Matter.Module.html#concept_title/1"},{"type":"function","title":"Example - Magma.Matter.Module.concept_title/1","doc":"iex> Some.Module\n  ...> |> Magma.Matter.Module.new!()\n  ...> |> Magma.Matter.Module.concept_title()\n  \"`Some.Module`\"","ref":"Magma.Matter.Module.html#concept_title/1-example"},{"type":"function","title":"Magma.Matter.Module.config/0","doc":"","ref":"Magma.Matter.Module.html#config/0"},{"type":"function","title":"Magma.Matter.Module.config/1","doc":"","ref":"Magma.Matter.Module.html#config/1"},{"type":"function","title":"Magma.Matter.Module.config_name/0","doc":"","ref":"Magma.Matter.Module.html#config_name/0"},{"type":"function","title":"Magma.Matter.Module.context_modules/1","doc":"Returns a list of the modules the given `module` is defined under.\n\n#","ref":"Magma.Matter.Module.html#context_modules/1"},{"type":"function","title":"Example - Magma.Matter.Module.context_modules/1","doc":"iex> Magma.Matter.Module.context_modules(Magma.DocumentStruct.Section)\n    [Magma, Magma.DocumentStruct]","ref":"Magma.Matter.Module.html#context_modules/1-example"},{"type":"function","title":"Magma.Matter.Module.default_description/2","doc":"Returns a default description for the `Magma.Concept` document of an Elixir module.","ref":"Magma.Matter.Module.html#default_description/2"},{"type":"function","title":"Magma.Matter.Module.ignore?/1","doc":"Determines whether the module should be ignored when generating documentation.\n\nA module is ignored\n\n- if it has a # Magma pragma: ignore comment at the beginning of its source code, or\n- if it is marked as hidden (e.g. with `@moduledoc false`) and does not have a\n  # Magma pragma: include comment at the beginning of its source code.","ref":"Magma.Matter.Module.html#ignore?/1"},{"type":"function","title":"Magma.Matter.Module.new/1","doc":"Creates a new `Magma.Matter.Module` instance from a given module name in an ok tuple.","ref":"Magma.Matter.Module.html#new/1"},{"type":"function","title":"Magma.Matter.Module.new!/1","doc":"Creates a new `Magma.Matter.Module` instance from a given module name and fails in error cases.","ref":"Magma.Matter.Module.html#new!/1"},{"type":"function","title":"Magma.Matter.Module.prompt_concept_description_title/1","doc":"Returns the title for the description section of the module in artefact prompts.\n\n#","ref":"Magma.Matter.Module.html#prompt_concept_description_title/1"},{"type":"function","title":"Example - Magma.Matter.Module.prompt_concept_description_title/1","doc":"iex> Some.Module\n    ...> |> Magma.Matter.Module.new!()\n    ...> |> Magma.Matter.Module.prompt_concept_description_title()\n    \"Description of the module `Some.Module`\"","ref":"Magma.Matter.Module.html#prompt_concept_description_title/1-example"},{"type":"function","title":"Magma.Matter.Module.prompt_matter_description/1","doc":"Returns a string with a Markdown section containing the source code of the module for artefact prompts.","ref":"Magma.Matter.Module.html#prompt_matter_description/1"},{"type":"function","title":"Magma.Matter.Module.relative_base_path/1","doc":"Returns the base path segment to be used for different kinds of documents for Elixir modules.\n\nThe base path for all modules is `\"modules\"`.","ref":"Magma.Matter.Module.html#relative_base_path/1"},{"type":"function","title":"Magma.Matter.Module.relative_concept_path/1","doc":"Returns the path for `Magma.Concept` documents about Elixir modules.\n\n#","ref":"Magma.Matter.Module.html#relative_concept_path/1"},{"type":"function","title":"Example - Magma.Matter.Module.relative_concept_path/1","doc":"iex> Some.Module\n    ...> |> Magma.Matter.Module.new!()\n    ...> |> Magma.Matter.Module.relative_concept_path()\n    \"modules/Some/Some.Module.md\"","ref":"Magma.Matter.Module.html#relative_concept_path/1-example"},{"type":"function","title":"Magma.Matter.Module.source_path/1","doc":"Returns the source path of the `module`, if it exists.\n\nThe source path is the file path where the source code of the module is located.","ref":"Magma.Matter.Module.html#source_path/1"},{"type":"function","title":"Magma.Matter.Module.submodules/1","doc":"Returns a list of the submodules defined under the given `module`.\n\nNote: This function relies on the existence of concept documents for modules.","ref":"Magma.Matter.Module.html#submodules/1"},{"type":"type","title":"Magma.Matter.Module.t/0","doc":"","ref":"Magma.Matter.Module.html#t:t/0"},{"type":"module","title":"Magma.Matter.Project","doc":"`Magma.Matter` type behaviour implementation for the project Magma is used for.\n\nIt is unique in the sense that there is only one instance of it, corresponding\nto the one project for which artefacts are being created. It plays a central\nrole as its description (in the corresponding `Magma.Concept` about this matter)\nis included in every prompt.\n\nThe single `Magma.Concept` for the project can be fetched with the `concept/0`\nfunction.","ref":"Magma.Matter.Project.html"},{"type":"function","title":"Magma.Matter.Project.app_name/0","doc":"Returns the project's app name as specified in the `mix.exs` file.","ref":"Magma.Matter.Project.html#app_name/0"},{"type":"function","title":"Magma.Matter.Project.artefacts/0","doc":"Returns the list of `Magma.Artefact` types available for a project.\n\n    iex> Magma.Matter.Project.artefacts()\n    [Magma.Artefacts.Readme]","ref":"Magma.Matter.Project.html#artefacts/0"},{"type":"function","title":"Magma.Matter.Project.concept/0","doc":"Returns the `Magma.Concept` about the project.","ref":"Magma.Matter.Project.html#concept/0"},{"type":"function","title":"Magma.Matter.Project.concept_name/1","doc":"Returns the name of the `Magma.Concept` document about the project.\n\nIn order to not get in name conflict with any other document (e.g. the\ndocument about the top-level module which is usually the project name),\nand there's only one such matter the project concept is generally called\n\"Project\".\n\n#","ref":"Magma.Matter.Project.html#concept_name/1"},{"type":"function","title":"Example - Magma.Matter.Project.concept_name/1","doc":"iex> \"Example\"\n    ...> |> Magma.Matter.Project.new!()\n    ...> |> Magma.Matter.Project.concept_name()\n    \"Project\"","ref":"Magma.Matter.Project.html#concept_name/1-example"},{"type":"function","title":"Magma.Matter.Project.concept_title/1","doc":"Returns the title header text of the `Magma.Concept` document about the project.\n\n#","ref":"Magma.Matter.Project.html#concept_title/1"},{"type":"function","title":"Example - Magma.Matter.Project.concept_title/1","doc":"iex> \"Example\"\n    ...> |> Magma.Matter.Project.new!()\n    ...> |> Magma.Matter.Project.concept_title()\n    \"Example project\"","ref":"Magma.Matter.Project.html#concept_title/1-example"},{"type":"function","title":"Magma.Matter.Project.config/0","doc":"","ref":"Magma.Matter.Project.html#config/0"},{"type":"function","title":"Magma.Matter.Project.config/1","doc":"","ref":"Magma.Matter.Project.html#config/1"},{"type":"function","title":"Magma.Matter.Project.config_name/0","doc":"","ref":"Magma.Matter.Project.html#config_name/0"},{"type":"function","title":"Magma.Matter.Project.default_concept_aliases/1","doc":"Returns a list of Obsidian aliases for the `Magma.Concept` document about the project.\n\n#","ref":"Magma.Matter.Project.html#default_concept_aliases/1"},{"type":"function","title":"Example - Magma.Matter.Project.default_concept_aliases/1","doc":"iex> \"Example\"\n    ...> |> Magma.Matter.Project.new!()\n    ...> |> Magma.Matter.Project.default_concept_aliases()\n    [\"Example project\", \"Example-project\"]","ref":"Magma.Matter.Project.html#default_concept_aliases/1-example"},{"type":"function","title":"Magma.Matter.Project.default_description/2","doc":"Returns a default description for the `Magma.Concept` about the project.","ref":"Magma.Matter.Project.html#default_description/2"},{"type":"function","title":"Magma.Matter.Project.extract_from_metadata/3","doc":"Extracts the project name from the metadata of a `Magma.Concept` document about the project and creates a new instance with it.\n\nThe project name must be specified in the `magma_matter_name` YAML frontmatter property.\nIf the project name is not found, it returns an error.","ref":"Magma.Matter.Project.html#extract_from_metadata/3"},{"type":"function","title":"Magma.Matter.Project.modules/0","doc":"Returns all modules of the project as `Magma.Matter.Module`s.\n\nwhich are not ignored in terms of `Magma.Matter.Module.ignore?/1`","ref":"Magma.Matter.Project.html#modules/0"},{"type":"function","title":"Magma.Matter.Project.new/1","doc":"Creates a new `Magma.Matter.Project` instance from the given name in an ok tuple.","ref":"Magma.Matter.Project.html#new/1"},{"type":"function","title":"Magma.Matter.Project.new!/1","doc":"Creates a new `Magma.Matter.Project` instance from the given name and fails in error cases.","ref":"Magma.Matter.Project.html#new!/1"},{"type":"function","title":"Magma.Matter.Project.prompt_concept_description_title/1","doc":"Returns the title for the description section of the project in artefact prompts.\n\n#","ref":"Magma.Matter.Project.html#prompt_concept_description_title/1"},{"type":"function","title":"Example - Magma.Matter.Project.prompt_concept_description_title/1","doc":"iex> \"Example\"\n    ...> |> Magma.Matter.Project.new!()\n    ...> |> Magma.Matter.Project.prompt_concept_description_title()\n    \"Description of the 'Example' project\"","ref":"Magma.Matter.Project.html#prompt_concept_description_title/1-example"},{"type":"function","title":"Magma.Matter.Project.relative_base_path/1","doc":"Returns the base path segment to be used for the document about the project.\n\nSince there is just one such matter, and it has a central, this base path is\nempty, meaning it these documents are placed all at the root of the folders\nfor the different document types.","ref":"Magma.Matter.Project.html#relative_base_path/1"},{"type":"function","title":"Magma.Matter.Project.relative_concept_path/1","doc":"Returns the path for `Magma.Concept` document about the project.\n\nSee also `concept_name/1`.\n\n#","ref":"Magma.Matter.Project.html#relative_concept_path/1"},{"type":"function","title":"Example - Magma.Matter.Project.relative_concept_path/1","doc":"iex> \"Example\"\n    ...> |> Magma.Matter.Project.new!()\n    ...> |> Magma.Matter.Project.relative_concept_path()\n    \"Project.md\"","ref":"Magma.Matter.Project.html#relative_concept_path/1-example"},{"type":"function","title":"Magma.Matter.Project.render_front_matter/1","doc":"Renders the YAML frontmatter properties specific for the `Magma.Concept` document about the project.\n\nIn particular this includes `magma_matter_name` with the project name.","ref":"Magma.Matter.Project.html#render_front_matter/1"},{"type":"function","title":"Magma.Matter.Project.version/0","doc":"Returns the project's version as specified in the `mix.exs` file.","ref":"Magma.Matter.Project.html#version/0"},{"type":"type","title":"Magma.Matter.Project.t/0","doc":"","ref":"Magma.Matter.Project.html#t:t/0"},{"type":"behaviour","title":"Magma.Artefact","doc":"`Magma.Artefact` is a behaviour for defining different types of artefacts.\n\nA Magma artefact represents a specific type of output that can be generated\nfor some Magma matter. The module provides a set of callbacks for specifying\nvarious aspects of the artefacts such as naming, path definitions, and\ndocument generation details.","ref":"Magma.Artefact.html"},{"type":"callback","title":"Magma.Artefact.concept_prompt_task_section_title/0","doc":"A callback that returns the title of the \"Artefacts\" subsection for this type of matter in the `Magma.Concept` document where for the text returned by the `c:request_prompt_task/1` callback is rendered.\n\nBy default, this is just the `concept_section_title/0` with `\"prompt task\"` appended.","ref":"Magma.Artefact.html#c:concept_prompt_task_section_title/0"},{"type":"callback","title":"Magma.Artefact.concept_section_title/0","doc":"A callback that returns the title of the \"Artefacts\" subsection for this type of matter in the `Magma.Concept` document.\n\nThis section consists of links to the `Magma.Artefact.Prompt` and the\n`Magma.Artefact.Version` of this document and another subsection for the\ntext returned by the `c:request_prompt_task/1` callback.","ref":"Magma.Artefact.html#c:concept_section_title/0"},{"type":"callback","title":"Magma.Artefact.context_knowledge/1","doc":"A callback that allows to specify texts which should be included generally in the \"Context knowledge\" section of the `Magma.Artefact.Prompt` document about this type of artefact.","ref":"Magma.Artefact.html#c:context_knowledge/1"},{"type":"callback","title":"Magma.Artefact.create_version/2","doc":"A callback that allows to implement a custom `Magma.Artefact.Version` document creation function.\n\nThis function should return `nil` if the default `Magma.Artefact.Version.create/2`\nshould be used (which the default implementation does automatically).","ref":"Magma.Artefact.html#c:create_version/2"},{"type":"callback","title":"Magma.Artefact.default_name/1","doc":"A callback that returns the name of an artefact to be used as default for the `:name` field.","ref":"Magma.Artefact.html#c:default_name/1"},{"type":"function","title":"Magma.Artefact.extract_from_metadata/1","doc":"Extracts an `Magma.Artefact` instance from YAML frontmatter metadata.\n\nThe function attempts to retrieve the `magma_artefact` and\n`magma_concept` from the metadata. It returns a tuple containing\nthe artefact (if found and valid), and the remaining metadata.","ref":"Magma.Artefact.html#extract_from_metadata/1"},{"type":"function","title":"Magma.Artefact.fields/0","doc":"","ref":"Magma.Artefact.html#fields/0"},{"type":"callback","title":"Magma.Artefact.new/1","doc":"A callback that creates a new instance of a type of artefact with the default name.","ref":"Magma.Artefact.html#c:new/1"},{"type":"callback","title":"Magma.Artefact.new/2","doc":"A callback that creates a new instance of a type of artefact.","ref":"Magma.Artefact.html#c:new/2"},{"type":"callback","title":"Magma.Artefact.prompt_name/1","doc":"A callback that returns the name of the `Magma.Artefact.Prompt` document for this type of matter.","ref":"Magma.Artefact.html#c:prompt_name/1"},{"type":"callback","title":"Magma.Artefact.relative_base_path/1","doc":"A callback that returns the general path segment to be used for documents for this type of artefact.","ref":"Magma.Artefact.html#c:relative_base_path/1"},{"type":"function","title":"Magma.Artefact.relative_prompt_path/1","doc":"","ref":"Magma.Artefact.html#relative_prompt_path/1"},{"type":"callback","title":"Magma.Artefact.relative_prompt_path/1","doc":"A callback that returns the path for `Magma.Artefact.Prompt` documents about this type of artefact.\n\nSince the `Magma.PromptResult` document are always stored in the subdirectory\nwhere the prompt are stored, this function also determines their path.\n\nThis path is relative to the `Magma.Vault.artefact_generation_path/0`.","ref":"Magma.Artefact.html#c:relative_prompt_path/1"},{"type":"function","title":"Magma.Artefact.relative_version_path/1","doc":"","ref":"Magma.Artefact.html#relative_version_path/1"},{"type":"callback","title":"Magma.Artefact.relative_version_path/1","doc":"A callback that returns the path for `Magma.Artefact.Version` documents about this type of artefact.\n\nThis path is relative to the `Magma.Vault.artefact_version_path/0`.","ref":"Magma.Artefact.html#c:relative_version_path/1"},{"type":"function","title":"Magma.Artefact.render_front_matter/1","doc":"","ref":"Magma.Artefact.html#render_front_matter/1"},{"type":"callback","title":"Magma.Artefact.request_prompt_task/1","doc":"A callback that returns the request prompt text of the `Magma.Artefact.Concept` document for this type of matter that describes what to generate.\n\nDespite returning also a general text like the `c:system_prompt_task/1`, this\none is included in the \"Artefacts\" section of the `Magma.Concept` document\n(and only transcluded in `Magma.Artefact.Prompt` document), so that the user\nhas a chance to adapt it for a specific instance of this artefact type.\n\nThe generated default implementation returns the content of the\n\"Task prompt\" section of the respective artefact config document,\nafter EEx evaluation with the bindings returned by `c:request_prompt_task_template_bindings/1`.","ref":"Magma.Artefact.html#c:request_prompt_task/1"},{"type":"callback","title":"Magma.Artefact.request_prompt_task_template_bindings/1","doc":"A callback that returns the bindings to be applied when rendering the `c:request_prompt_task/1` EEx template.","ref":"Magma.Artefact.html#c:request_prompt_task_template_bindings/1"},{"type":"callback","title":"Magma.Artefact.system_prompt_task/1","doc":"A callback that returns the system prompt text of the `Magma.Artefact.Prompt` document for this type of matter that describes what to generate.\n\nAs opposed to the `c:request_prompt_task/1` this is a general, static text\nused by artefacts of this type.\n\nThe generated default implementation returns a transclusion of the\n\"System prompt\" section of the respective artefact config document.","ref":"Magma.Artefact.html#c:system_prompt_task/1"},{"type":"callback","title":"Magma.Artefact.trim_prompt_result_header?/0","doc":"A callback that returns if the initial header of a generated `Magma.PromptResult` for this type artefact should be stripped.\n\nSince the title for the `Magma.PromptResult` is already defined,\nthe title generated by an LLM should be ignored usually.\nFor some types of artefacts, however, this should not be the case.\nThese artefact types, the default implementation returning `true`,\ncan be overwritten.","ref":"Magma.Artefact.html#c:trim_prompt_result_header?/0"},{"type":"function","title":"Magma.Artefact.type/1","doc":"Returns the artefact type module for the given string.","ref":"Magma.Artefact.html#type/1"},{"type":"function","title":"Example - Magma.Artefact.type/1","doc":"iex> Magma.Artefact.type(\"ModuleDoc\")\n    Magma.Artefacts.ModuleDoc\n\n    iex> Magma.Artefact.type(\"TableOfContents\")\n    Magma.Artefacts.TableOfContents\n\n    iex> Magma.Artefact.type(\"Vault\")\n    nil\n\n    iex> Magma.Artefact.type(\"NonExisting\")\n    nil","ref":"Magma.Artefact.html#type/1-example"},{"type":"function","title":"Magma.Artefact.type?/1","doc":"Checks if the given `module` is a `Magma.Artefact` type module.","ref":"Magma.Artefact.html#type?/1"},{"type":"function","title":"Magma.Artefact.type_name/2","doc":"Returns the artefact type name for the given artefact type module.","ref":"Magma.Artefact.html#type_name/2"},{"type":"function","title":"Example - Magma.Artefact.type_name/2","doc":"iex> Magma.Artefact.type_name(Magma.Artefacts.ModuleDoc)\n    \"ModuleDoc\"\n\n    iex> Magma.Artefact.type_name(Magma.Artefacts.Article)\n    \"Article\"\n\n    iex> Magma.Artefact.type_name(Magma.Vault)\n    ** (RuntimeError) Invalid Magma.Artefacts type: Magma.Vault\n\n    iex> Magma.Artefact.type_name(NonExisting)\n    ** (RuntimeError) Invalid Magma.Artefacts type: NonExisting","ref":"Magma.Artefact.html#type_name/2-example"},{"type":"callback","title":"Magma.Artefact.version_prologue/1","doc":"A callback that allows to specify a text which should be included in the prologue of the `Magma.Artefact.Version` document of this artefact type.","ref":"Magma.Artefact.html#c:version_prologue/1"},{"type":"callback","title":"Magma.Artefact.version_title/1","doc":"A callback that returns the title to be used for the `Magma.Artefact.Version` document.","ref":"Magma.Artefact.html#c:version_title/1"},{"type":"type","title":"Magma.Artefact.t/0","doc":"","ref":"Magma.Artefact.html#t:t/0"},{"type":"type","title":"Magma.Artefact.type/0","doc":"","ref":"Magma.Artefact.html#t:type/0"},{"type":"module","title":"Magma.Artefacts.Article","doc":"","ref":"Magma.Artefacts.Article.html"},{"type":"function","title":"Magma.Artefacts.Article.config/0","doc":"","ref":"Magma.Artefacts.Article.html#config/0"},{"type":"function","title":"Magma.Artefacts.Article.config/1","doc":"","ref":"Magma.Artefacts.Article.html#config/1"},{"type":"function","title":"Magma.Artefacts.Article.config_name/0","doc":"","ref":"Magma.Artefacts.Article.html#config_name/0"},{"type":"function","title":"Magma.Artefacts.Article.matter_type/0","doc":"","ref":"Magma.Artefacts.Article.html#matter_type/0"},{"type":"function","title":"Magma.Artefacts.Article.new!/2","doc":"","ref":"Magma.Artefacts.Article.html#new!/2"},{"type":"module","title":"Magma.Artefacts.ModuleDoc","doc":"","ref":"Magma.Artefacts.ModuleDoc.html"},{"type":"function","title":"Magma.Artefacts.ModuleDoc.config/0","doc":"","ref":"Magma.Artefacts.ModuleDoc.html#config/0"},{"type":"function","title":"Magma.Artefacts.ModuleDoc.config/1","doc":"","ref":"Magma.Artefacts.ModuleDoc.html#config/1"},{"type":"function","title":"Magma.Artefacts.ModuleDoc.config_name/0","doc":"","ref":"Magma.Artefacts.ModuleDoc.html#config_name/0"},{"type":"function","title":"Magma.Artefacts.ModuleDoc.get/1","doc":"","ref":"Magma.Artefacts.ModuleDoc.html#get/1"},{"type":"function","title":"Magma.Artefacts.ModuleDoc.matter_type/0","doc":"","ref":"Magma.Artefacts.ModuleDoc.html#matter_type/0"},{"type":"function","title":"Magma.Artefacts.ModuleDoc.new!/2","doc":"","ref":"Magma.Artefacts.ModuleDoc.html#new!/2"},{"type":"function","title":"Magma.Artefacts.ModuleDoc.prompt_result_section_title/0","doc":"","ref":"Magma.Artefacts.ModuleDoc.html#prompt_result_section_title/0"},{"type":"function","title":"Magma.Artefacts.ModuleDoc.version_path/1","doc":"","ref":"Magma.Artefacts.ModuleDoc.html#version_path/1"},{"type":"module","title":"Magma.Artefacts.Readme","doc":"","ref":"Magma.Artefacts.Readme.html"},{"type":"function","title":"Magma.Artefacts.Readme.config/0","doc":"","ref":"Magma.Artefacts.Readme.html#config/0"},{"type":"function","title":"Magma.Artefacts.Readme.config/1","doc":"","ref":"Magma.Artefacts.Readme.html#config/1"},{"type":"function","title":"Magma.Artefacts.Readme.config_name/0","doc":"","ref":"Magma.Artefacts.Readme.html#config_name/0"},{"type":"function","title":"Magma.Artefacts.Readme.matter_type/0","doc":"","ref":"Magma.Artefacts.Readme.html#matter_type/0"},{"type":"function","title":"Magma.Artefacts.Readme.new!/2","doc":"","ref":"Magma.Artefacts.Readme.html#new!/2"},{"type":"module","title":"Magma.Artefacts.TableOfContents","doc":"","ref":"Magma.Artefacts.TableOfContents.html"},{"type":"function","title":"Magma.Artefacts.TableOfContents.assemble_button/0","doc":"","ref":"Magma.Artefacts.TableOfContents.html#assemble_button/0"},{"type":"function","title":"Magma.Artefacts.TableOfContents.assemble_callout/1","doc":"","ref":"Magma.Artefacts.TableOfContents.html#assemble_callout/1"},{"type":"function","title":"Magma.Artefacts.TableOfContents.config/0","doc":"","ref":"Magma.Artefacts.TableOfContents.html#config/0"},{"type":"function","title":"Magma.Artefacts.TableOfContents.config/1","doc":"","ref":"Magma.Artefacts.TableOfContents.html#config/1"},{"type":"function","title":"Magma.Artefacts.TableOfContents.config_name/0","doc":"","ref":"Magma.Artefacts.TableOfContents.html#config_name/0"},{"type":"function","title":"Magma.Artefacts.TableOfContents.matter_type/0","doc":"","ref":"Magma.Artefacts.TableOfContents.html#matter_type/0"},{"type":"function","title":"Magma.Artefacts.TableOfContents.new!/2","doc":"","ref":"Magma.Artefacts.TableOfContents.html#new!/2"},{"type":"module","title":"Magma.Matter.Text","doc":"","ref":"Magma.Matter.Text.html"},{"type":"function","title":"Magma.Matter.Text.config/0","doc":"","ref":"Magma.Matter.Text.html#config/0"},{"type":"function","title":"Magma.Matter.Text.config/1","doc":"","ref":"Magma.Matter.Text.html#config/1"},{"type":"function","title":"Magma.Matter.Text.config_name/0","doc":"","ref":"Magma.Matter.Text.html#config_name/0"},{"type":"function","title":"Magma.Matter.Text.new/2","doc":"","ref":"Magma.Matter.Text.html#new/2"},{"type":"function","title":"Magma.Matter.Text.new!/2","doc":"","ref":"Magma.Matter.Text.html#new!/2"},{"type":"function","title":"Magma.Matter.Text.sections_section_title/0","doc":"","ref":"Magma.Matter.Text.html#sections_section_title/0"},{"type":"function","title":"Magma.Matter.Text.type/2","doc":"Returns the text type module for the given string.","ref":"Magma.Matter.Text.html#type/2"},{"type":"function","title":"Example - Magma.Matter.Text.type/2","doc":"iex> Magma.Matter.Text.type(\"UserGuide\")\n    Magma.Matter.Texts.UserGuide\n\n    iex> Magma.Matter.Text.type(\"Generic\")\n    Magma.Matter.Texts.Generic\n\n    iex> Magma.Matter.Text.type(\"Vault\")\n    nil\n\n    iex> Magma.Matter.Text.type(\"NonExisting\")\n    nil","ref":"Magma.Matter.Text.html#type/2-example"},{"type":"function","title":"Magma.Matter.Text.type?/1","doc":"","ref":"Magma.Matter.Text.html#type?/1"},{"type":"function","title":"Magma.Matter.Text.type_name/2","doc":"Returns the text type name for the given text module.","ref":"Magma.Matter.Text.html#type_name/2"},{"type":"function","title":"Example - Magma.Matter.Text.type_name/2","doc":"iex> Magma.Matter.Text.type_name(Magma.Matter.Texts.UserGuide)\n    \"UserGuide\"\n\n    iex> Magma.Matter.Text.type_name(Magma.Matter.Texts.Generic)\n    \"Generic\"\n\n    iex> Magma.Matter.Text.type_name(Magma.Vault)\n    ** (RuntimeError) Invalid Magma.Matter.Text type: Magma.Vault\n\n    iex> Magma.Matter.Text.type_name(NonExisting)\n    ** (RuntimeError) Invalid Magma.Matter.Text type: NonExisting","ref":"Magma.Matter.Text.html#type_name/2-example"},{"type":"type","title":"Magma.Matter.Text.t/0","doc":"","ref":"Magma.Matter.Text.html#t:t/0"},{"type":"type","title":"Magma.Matter.Text.type/0","doc":"","ref":"Magma.Matter.Text.html#t:type/0"},{"type":"module","title":"Magma.Matter.Text.Section","doc":"","ref":"Magma.Matter.Text.Section.html"},{"type":"function","title":"Magma.Matter.Text.Section.config/0","doc":"","ref":"Magma.Matter.Text.Section.html#config/0"},{"type":"function","title":"Magma.Matter.Text.Section.config/1","doc":"","ref":"Magma.Matter.Text.Section.html#config/1"},{"type":"function","title":"Magma.Matter.Text.Section.config_name/0","doc":"","ref":"Magma.Matter.Text.Section.html#config_name/0"},{"type":"function","title":"Magma.Matter.Text.Section.new/1","doc":"","ref":"Magma.Matter.Text.Section.html#new/1"},{"type":"function","title":"Magma.Matter.Text.Section.new/2","doc":"","ref":"Magma.Matter.Text.Section.html#new/2"},{"type":"function","title":"Magma.Matter.Text.Section.new!/1","doc":"","ref":"Magma.Matter.Text.Section.html#new!/1"},{"type":"function","title":"Magma.Matter.Text.Section.new!/2","doc":"","ref":"Magma.Matter.Text.Section.html#new!/2"},{"type":"type","title":"Magma.Matter.Text.Section.t/0","doc":"","ref":"Magma.Matter.Text.Section.html#t:t/0"},{"type":"module","title":"Magma.Text","doc":"","ref":"Magma.Text.html"},{"type":"function","title":"Magma.Text.create/2","doc":"","ref":"Magma.Text.html#create/2"},{"type":"module","title":"Magma.Text.Preview","doc":"","ref":"Magma.Text.Preview.html"},{"type":"function","title":"Magma.Text.Preview.build_name/1","doc":"","ref":"Magma.Text.Preview.html#build_name/1"},{"type":"function","title":"Magma.Text.Preview.create/3","doc":"","ref":"Magma.Text.Preview.html#create/3"},{"type":"function","title":"Magma.Text.Preview.dir/0","doc":"","ref":"Magma.Text.Preview.html#dir/0"},{"type":"function","title":"Magma.Text.Preview.from!/1","doc":"Fetches a document from a related document and immediately loads it with `load!/1`","ref":"Magma.Text.Preview.html#from!/1"},{"type":"function","title":"Magma.Text.Preview.load/1","doc":"Loads a document from the given `path` or `document`.\n\nIf the loaded document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Text.Preview.html#load/1"},{"type":"function","title":"Magma.Text.Preview.load!/1","doc":"Loads a document from the given `path` or `document` and raises an exception in error cases.","ref":"Magma.Text.Preview.html#load!/1"},{"type":"function","title":"Magma.Text.Preview.load_linked/1","doc":"Loads a document from the given Obsidian `link`, i.e. a string of the form `\"[[name]]\"`.\n\nIf the referenced document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Text.Preview.html#load_linked/1"},{"type":"function","title":"Magma.Text.Preview.new/2","doc":"","ref":"Magma.Text.Preview.html#new/2"},{"type":"function","title":"Magma.Text.Preview.new!/2","doc":"","ref":"Magma.Text.Preview.html#new!/2"},{"type":"function","title":"Magma.Text.Preview.prologue/0","doc":"","ref":"Magma.Text.Preview.html#prologue/0"},{"type":"function","title":"Magma.Text.Preview.render/1","doc":"","ref":"Magma.Text.Preview.html#render/1"},{"type":"type","title":"Magma.Text.Preview.t/0","doc":"","ref":"Magma.Text.Preview.html#t:t/0"},{"type":"behaviour","title":"Magma.Generation","doc":"Generic adapter-based `Magma.Prompt` execution.\n\nThe `Magma.Generation` module is primarily responsible for handling the\nexecution of prompts. It is designed to be adaptable and flexible,\nsupporting different LLMs via specific adapters.\nThe module defines a behaviour that each adapter should implement,\nensuring a consistent interface for executing prompts.\n\nThe currently implemented adapters are:\n\n- `Magma.Generation.OpenAI` for the OpenAI API\n- `Magma.Generation.Manual` for manual prompt execution\n\nThe default values for the generation specification embedded within a prompt\ndocument (the `magma_generation_type` and `magma_generation_params` properties\nin its YAML frontmatter) can be configured for your application in `config.exs`\nlike this:\n\n    config :magma,\n      default_generation: Magma.Generation.OpenAI\n\n    config :magma, Magma.Generation.OpenAI,\n      model: \"gpt-4\",\n      temperature: 0.6\n\nExcept within the `:test` environment, the defaults can be configured also\nwith the `default_generation_type` and `:default_generation_params` properties\nin YAML frontmatter of the `magma_config.md` document in your vault, taking\nprecedence over the ones from the application config.\n\nUnlike, the default generation params from the `magma_config.md` document,\nthe ones from the application config are used also as initial defaults on the\n`new/1` function of a `Magma.Generation` implementation, meaning you only\nhave to provide the values differing from the application configured ones.","ref":"Magma.Generation.html"},{"type":"function","title":"Magma.Generation.default/0","doc":"","ref":"Magma.Generation.html#default/0"},{"type":"callback","title":"Magma.Generation.default_params/0","doc":"","ref":"Magma.Generation.html#c:default_params/0"},{"type":"function","title":"Magma.Generation.execute/1","doc":"","ref":"Magma.Generation.html#execute/1"},{"type":"function","title":"Magma.Generation.execute/3","doc":"","ref":"Magma.Generation.html#execute/3"},{"type":"callback","title":"Magma.Generation.execute/3","doc":"","ref":"Magma.Generation.html#c:execute/3"},{"type":"function","title":"Magma.Generation.extract_from_metadata/1","doc":"Extracts generation information from YAML frontmatter metadata.\n\nThe function attempts to retrieve the `magma_generation_type` and\n`magma_generation_params` from the metadata. It returns a tuple containing\nthe generation (if found and valid), and the remaining metadata.","ref":"Magma.Generation.html#extract_from_metadata/1"},{"type":"function","title":"Magma.Generation.render_front_matter/1","doc":"Renders generation YAML frontmatter properties.","ref":"Magma.Generation.html#render_front_matter/1"},{"type":"function","title":"Magma.Generation.short_name/1","doc":"Returns the short version of the `Magma.Generation` implementation name.\n\nThis is used as the `magma_generation` value in the YAML frontmatter.","ref":"Magma.Generation.html#short_name/1"},{"type":"function","title":"Example - Magma.Generation.short_name/1","doc":"iex> Magma.Generation.short_name(Magma.Generation.OpenAI)\n    OpenAI\n\n    iex> Magma.Generation.short_name(Magma.Generation.Bumblebee.TextGeneration.Llama)\n    Bumblebee.TextGeneration.Llama","ref":"Magma.Generation.html#short_name/1-example"},{"type":"function","title":"Magma.Generation.type/1","doc":"Returns the generation module for the given string.","ref":"Magma.Generation.html#type/1"},{"type":"function","title":"Example - Magma.Generation.type/1","doc":"iex> Magma.Generation.type(\"OpenAI\")\n    Magma.Generation.OpenAI\n\n    iex> Magma.Generation.type(\"Manual\")\n    Magma.Generation.Manual\n\n    iex> Magma.Generation.type(\"Mock\")\n    Magma.Generation.Mock\n\n    iex> Magma.Generation.type(\"Vault\")\n    nil\n\n    iex> Magma.Generation.type(\"NonExisting\")\n    nil","ref":"Magma.Generation.html#type/1-example"},{"type":"type","title":"Magma.Generation.options/0","doc":"","ref":"Magma.Generation.html#t:options/0"},{"type":"type","title":"Magma.Generation.prompt/0","doc":"","ref":"Magma.Generation.html#t:prompt/0"},{"type":"type","title":"Magma.Generation.result/0","doc":"","ref":"Magma.Generation.html#t:result/0"},{"type":"type","title":"Magma.Generation.system_prompt/0","doc":"","ref":"Magma.Generation.html#t:system_prompt/0"},{"type":"type","title":"Magma.Generation.t/0","doc":"","ref":"Magma.Generation.html#t:t/0"},{"type":"module","title":"Magma.Generation.Manual","doc":"","ref":"Magma.Generation.Manual.html"},{"type":"function","title":"Magma.Generation.Manual.new/1","doc":"","ref":"Magma.Generation.Manual.html#new/1"},{"type":"function","title":"Magma.Generation.Manual.new!/1","doc":"","ref":"Magma.Generation.Manual.html#new!/1"},{"type":"module","title":"Magma.Generation.OpenAI","doc":"","ref":"Magma.Generation.OpenAI.html"},{"type":"function","title":"Magma.Generation.OpenAI.new/1","doc":"","ref":"Magma.Generation.OpenAI.html#new/1"},{"type":"function","title":"Magma.Generation.OpenAI.new!/1","doc":"","ref":"Magma.Generation.OpenAI.html#new!/1"},{"type":"module","title":"Magma.Config","doc":"Cache for the vault-based configuration.\n\nThis singleton GenServer provides access to the cached\n`Magma.Config.Document`s in the `Magma.Vault`.","ref":"Magma.Config.html"},{"type":"function","title":"Magma.Config.artefact/2","doc":"Returns a `Magma.Config.Artefact` document or a property of it.","ref":"Magma.Config.html#artefact/2"},{"type":"function","title":"Magma.Config.artefacts_path/0","doc":"Returns the path with `Magma.Config.Artefact` documents in the vault.","ref":"Magma.Config.html#artefacts_path/0"},{"type":"function","title":"Magma.Config.artefacts_path/1","doc":"Constructs a complete path to a artefact config document by joining the specified `segments` to the `artefacts_path/0`.","ref":"Magma.Config.html#artefacts_path/1"},{"type":"function","title":"Magma.Config.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Magma.Config.html#child_spec/1"},{"type":"function","title":"Magma.Config.matter/2","doc":"Returns a `Magma.Config.Matter` document or a property of it.","ref":"Magma.Config.html#matter/2"},{"type":"function","title":"Magma.Config.matter_path/0","doc":"Returns the path with `Magma.Config.Matter` documents in the vault.","ref":"Magma.Config.html#matter_path/0"},{"type":"function","title":"Magma.Config.matter_path/1","doc":"Constructs a complete path to a matter config document by joining the specified `segments` to the `matter_path/0`.","ref":"Magma.Config.html#matter_path/1"},{"type":"function","title":"Magma.Config.path/0","doc":"Returns the path with `Magma.Config.Document`s in the vault.","ref":"Magma.Config.html#path/0"},{"type":"function","title":"Magma.Config.path/1","doc":"Constructs a complete path to a config document by joining the specified `segments` to the `path/0`.","ref":"Magma.Config.html#path/1"},{"type":"function","title":"Magma.Config.project/1","doc":"Returns the `Magma.Project` document or a property of it.","ref":"Magma.Config.html#project/1"},{"type":"function","title":"Magma.Config.reset/0","doc":"Resets the cache of all documents.","ref":"Magma.Config.html#reset/0"},{"type":"function","title":"Magma.Config.start_link/1","doc":"","ref":"Magma.Config.html#start_link/1"},{"type":"function","title":"Magma.Config.system/1","doc":"Returns the `Magma.Config.System` document or a property of it.","ref":"Magma.Config.html#system/1"},{"type":"function","title":"Magma.Config.template_path/0","doc":"Returns the path with templates for the config document for a new vault.","ref":"Magma.Config.html#template_path/0"},{"type":"function","title":"Magma.Config.text_type/2","doc":"Returns a `Magma.Config.TextType` document or a property of it.","ref":"Magma.Config.html#text_type/2"},{"type":"function","title":"Magma.Config.text_types/0","doc":"Returns a list of all text types.","ref":"Magma.Config.html#text_types/0"},{"type":"function","title":"Magma.Config.text_types_path/0","doc":"Returns the path with `Magma.Config.TextType` documents in the vault.","ref":"Magma.Config.html#text_types_path/0"},{"type":"function","title":"Magma.Config.text_types_path/1","doc":"Constructs a complete path to a text type config document by joining the specified `segments` to the `text_types_path/0`.","ref":"Magma.Config.html#text_types_path/1"},{"type":"type","title":"Magma.Config.t/0","doc":"","ref":"Magma.Config.html#t:t/0"},{"type":"module","title":"Magma.Config.Artefact","doc":"`Magma.Config.Document` for `Magma.Artefact` types.","ref":"Magma.Config.Artefact.html"},{"type":"function","title":"Magma.Config.Artefact.context_knowledge_transclusion/1","doc":"","ref":"Magma.Config.Artefact.html#context_knowledge_transclusion/1"},{"type":"function","title":"Magma.Config.Artefact.from!/1","doc":"Fetches a document from a related document and immediately loads it with `load!/1`","ref":"Magma.Config.Artefact.html#from!/1"},{"type":"function","title":"Magma.Config.Artefact.load/1","doc":"Loads a document from the given `path` or `document`.\n\nIf the loaded document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Config.Artefact.html#load/1"},{"type":"function","title":"Magma.Config.Artefact.load!/1","doc":"Loads a document from the given `path` or `document` and raises an exception in error cases.","ref":"Magma.Config.Artefact.html#load!/1"},{"type":"function","title":"Magma.Config.Artefact.load_linked/1","doc":"Loads a document from the given Obsidian `link`, i.e. a string of the form `\"[[name]]\"`.\n\nIf the referenced document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Config.Artefact.html#load_linked/1"},{"type":"function","title":"Magma.Config.Artefact.name_by_type/1","doc":"","ref":"Magma.Config.Artefact.html#name_by_type/1"},{"type":"function","title":"Magma.Config.Artefact.render_request_prompt/2","doc":"","ref":"Magma.Config.Artefact.html#render_request_prompt/2"},{"type":"function","title":"Magma.Config.Artefact.system_prompt_section_title/0","doc":"","ref":"Magma.Config.Artefact.html#system_prompt_section_title/0"},{"type":"function","title":"Magma.Config.Artefact.task_prompt_section_title/0","doc":"","ref":"Magma.Config.Artefact.html#task_prompt_section_title/0"},{"type":"module","title":"Magma.Config.Document","doc":"Foundational behaviors for Magma configuration documents.\n\nThis module is a base module that defines shared behavior and attributes for\nMagma configuration documents. It is used to ensure consistency and\nstandardization of configuration documents.","ref":"Magma.Config.Document.html"},{"type":"function","title":"Magma.Config.Document.context_knowledge_section_title/0","doc":"The title for the context knowledge section in configuration documents.","ref":"Magma.Config.Document.html#context_knowledge_section_title/0"},{"type":"function","title":"Magma.Config.Document.default_tags/0","doc":"Returns the default tags for Magma configuration documents.","ref":"Magma.Config.Document.html#default_tags/0"},{"type":"function","title":"Magma.Config.Document.init/1","doc":"Initializes a new Magma configuration document.","ref":"Magma.Config.Document.html#init/1"},{"type":"module","title":"Magma.Config.Matter","doc":"`Magma.Config.Document` for `Magma.Matter` types.","ref":"Magma.Config.Matter.html"},{"type":"function","title":"Magma.Config.Matter.context_knowledge_transclusion/1","doc":"","ref":"Magma.Config.Matter.html#context_knowledge_transclusion/1"},{"type":"function","title":"Magma.Config.Matter.from!/1","doc":"Fetches a document from a related document and immediately loads it with `load!/1`","ref":"Magma.Config.Matter.html#from!/1"},{"type":"function","title":"Magma.Config.Matter.load/1","doc":"Loads a document from the given `path` or `document`.\n\nIf the loaded document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Config.Matter.html#load/1"},{"type":"function","title":"Magma.Config.Matter.load!/1","doc":"Loads a document from the given `path` or `document` and raises an exception in error cases.","ref":"Magma.Config.Matter.html#load!/1"},{"type":"function","title":"Magma.Config.Matter.load_linked/1","doc":"Loads a document from the given Obsidian `link`, i.e. a string of the form `\"[[name]]\"`.\n\nIf the referenced document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Config.Matter.html#load_linked/1"},{"type":"function","title":"Magma.Config.Matter.name_by_type/1","doc":"","ref":"Magma.Config.Matter.html#name_by_type/1"},{"type":"module","title":"Magma.Config.System","doc":"`Magma.Config.Document` for the general Magma system configuration.","ref":"Magma.Config.System.html"},{"type":"function","title":"Magma.Config.System.context_knowledge_transclusion/0","doc":"","ref":"Magma.Config.System.html#context_knowledge_transclusion/0"},{"type":"function","title":"Magma.Config.System.default_generation/0","doc":"","ref":"Magma.Config.System.html#default_generation/0"},{"type":"function","title":"Magma.Config.System.default_link_resolution_style/0","doc":"","ref":"Magma.Config.System.html#default_link_resolution_style/0"},{"type":"function","title":"Magma.Config.System.default_persona/1","doc":"","ref":"Magma.Config.System.html#default_persona/1"},{"type":"function","title":"Magma.Config.System.default_tags/0","doc":"","ref":"Magma.Config.System.html#default_tags/0"},{"type":"function","title":"Magma.Config.System.from!/1","doc":"Fetches a document from a related document and immediately loads it with `load!/1`","ref":"Magma.Config.System.html#from!/1"},{"type":"function","title":"Magma.Config.System.load/0","doc":"","ref":"Magma.Config.System.html#load/0"},{"type":"function","title":"Magma.Config.System.load/1","doc":"Loads a document from the given `path` or `document`.\n\nIf the loaded document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Config.System.html#load/1"},{"type":"function","title":"Magma.Config.System.load!/1","doc":"Loads a document from the given `path` or `document` and raises an exception in error cases.","ref":"Magma.Config.System.html#load!/1"},{"type":"function","title":"Magma.Config.System.load_linked/1","doc":"Loads a document from the given Obsidian `link`, i.e. a string of the form `\"[[name]]\"`.\n\nIf the referenced document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Config.System.html#load_linked/1"},{"type":"function","title":"Magma.Config.System.name/0","doc":"","ref":"Magma.Config.System.html#name/0"},{"type":"function","title":"Magma.Config.System.new/1","doc":"","ref":"Magma.Config.System.html#new/1"},{"type":"function","title":"Magma.Config.System.new!/1","doc":"","ref":"Magma.Config.System.html#new!/1"},{"type":"function","title":"Magma.Config.System.path/0","doc":"","ref":"Magma.Config.System.html#path/0"},{"type":"function","title":"Magma.Config.System.persona_section_title/0","doc":"","ref":"Magma.Config.System.html#persona_section_title/0"},{"type":"function","title":"Magma.Config.System.persona_transclusion/0","doc":"","ref":"Magma.Config.System.html#persona_transclusion/0"},{"type":"function","title":"Magma.Config.System.template/1","doc":"","ref":"Magma.Config.System.html#template/1"},{"type":"function","title":"Magma.Config.System.title/0","doc":"","ref":"Magma.Config.System.html#title/0"},{"type":"type","title":"Magma.Config.System.t/0","doc":"","ref":"Magma.Config.System.html#t:t/0"},{"type":"module","title":"Magma.Config.TextType","doc":"`Magma.Config.Document` for `Magma.Matter.Text` types.","ref":"Magma.Config.TextType.html"},{"type":"function","title":"Magma.Config.TextType.context_knowledge_transclusion/1","doc":"","ref":"Magma.Config.TextType.html#context_knowledge_transclusion/1"},{"type":"function","title":"Magma.Config.TextType.create/3","doc":"","ref":"Magma.Config.TextType.html#create/3"},{"type":"function","title":"Magma.Config.TextType.from!/1","doc":"Fetches a document from a related document and immediately loads it with `load!/1`","ref":"Magma.Config.TextType.html#from!/1"},{"type":"function","title":"Magma.Config.TextType.load/1","doc":"Loads a document from the given `path` or `document`.\n\nIf the loaded document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Config.TextType.html#load/1"},{"type":"function","title":"Magma.Config.TextType.load!/1","doc":"Loads a document from the given `path` or `document` and raises an exception in error cases.","ref":"Magma.Config.TextType.html#load!/1"},{"type":"function","title":"Magma.Config.TextType.load_linked/1","doc":"Loads a document from the given Obsidian `link`, i.e. a string of the form `\"[[name]]\"`.\n\nIf the referenced document is not of the proper document type an `Magma.InvalidDocumentType`\nexception is returned in an `:error` tuple.","ref":"Magma.Config.TextType.html#load_linked/1"},{"type":"function","title":"Magma.Config.TextType.name_by_type/1","doc":"","ref":"Magma.Config.TextType.html#name_by_type/1"},{"type":"function","title":"Magma.Config.TextType.new/2","doc":"","ref":"Magma.Config.TextType.html#new/2"},{"type":"function","title":"Magma.Config.TextType.new!/2","doc":"","ref":"Magma.Config.TextType.html#new!/2"},{"type":"function","title":"Magma.Config.TextType.system_prompt_section_title/0","doc":"","ref":"Magma.Config.TextType.html#system_prompt_section_title/0"},{"type":"exception","title":"Magma.DocumentNotFound","doc":"Represents a missing document that is referenced somewhere.","ref":"Magma.DocumentNotFound.html"},{"type":"function","title":"Magma.DocumentNotFound.message/1","doc":"","ref":"Magma.DocumentNotFound.html#message/1"},{"type":"exception","title":"Magma.InvalidDocumentType","doc":"Raised when a document type does not match the expected one.","ref":"Magma.InvalidDocumentType.html"},{"type":"function","title":"Magma.InvalidDocumentType.message/1","doc":"","ref":"Magma.InvalidDocumentType.html#message/1"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\nThis project adheres to [Semantic Versioning](http://semver.org/) and\n[Keep a CHANGELOG](http://keepachangelog.com).","ref":"changelog.html"},{"type":"extras","title":"v0.2.0 - 2023-12-15 - Changelog","doc":"#","ref":"changelog.html#v0-2-0-2023-12-15"},{"type":"extras","title":"Added - Changelog","doc":"- Mix task `Mix.Tasks.Magma.Vault.Migrate` (`magma.vault.migrate`) to migrate a\n  vault to a newer version\n- Mix task `Mix.Tasks.Magma.Text.Type.New` (`magma.text.type.new`) to add\n  new custom text types.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- The configuration was moved into special config documents in the vault.\n  These include in particular also the system prompts and default task prompts,\n  which means they can now be easily adopted without having to touch any\n  Elixir code.\n- `Magma.Artefact` types are structs now, in order to support use cases\n  where multiple artefact instances of the same type for one concept should\n  be supported\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- encoding issues with the \"Copy to clipboard\" button when the prompt contained \n  special characters \n\n[Compare v0.1.1...v0.2.0](https://github.com/marcelotto/magma/compare/v0.1.1...v0.2.0)","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.1.1 - 2023-11-03 - Changelog","doc":"#","ref":"changelog.html#v0-1-1-2023-11-03"},{"type":"extras","title":"Fixed - Changelog","doc":"- a regression of the `Mix.Tasks.Magma.Prompt.Copy` Mix task\n\n[Compare v0.1.0...v0.1.1](https://github.com/marcelotto/magma/compare/v0.1.0...v0.1.1)","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.1.0 - 2023-11-03 - Changelog","doc":"Initial release","ref":"changelog.html#v0-1-0-2023-11-03"},{"type":"extras","title":"Introduction","doc":"<!-- ExDoc doesn't support YAML frontmatter\n\n---\nmagma_type: Artefact.Version\nmagma_artefact: Article\nmagma_concept: \"[[Magma User Guide - Introduction to Magma]]\"\nmagma_draft: \"[[Generated Magma User Guide - Introduction to Magma (article section) (2023-11-01T03:46:14)]]\"\ncreated_at: 2023-11-01 16:09:41\ntags: [magma-vault]\naliases: []\n---\n\n-->\n\n# Introduction\n\nMagma is an environment designed to support developers in writing and executing complex prompts. Although it's primarily targeted at Elixir developers, the system's core concept of prompt composition via transclusions makes it a powerful tool even outside this context. Magma also provides a solution when you need to generate longer and more complex texts. \n\nThe project was born out of the need to overcome the limitations of simple chat interfaces, especially when using Language Models (LLMs) such as ChatGPT for prompts about complex software projects or for writing extensive texts. Some of the challenges include:\n\n-   Limited editor capabilities of chat interfaces\n-   The cumbersome process of composing prompts by copying and pasting project knowledge repeatedly, leading to outdated and duplicated fragments\n-   The lack of efficient methods to manage and organize knowledge in chat interfaces\n\nMagma addresses these issues by providing a comprehensive environment for project knowledge bases and prompt development. It capitalizes on [Obsidian](https://obsidian.md/), a versatile tool for knowledge management, that works on top of a local folder of plain text Markdown files. Obsidian serves as the user interface for managing project knowledge and as an editor for producing documentation from the contained knowledge. For more details about Obsidian, please visit the [Obsidian Help site](https://help.obsidian.md/Home).\n\nAlthough Magma is designed for use with Obsidian, it's worth noting that since Obsidian documents are essentially Markdown documents, Magma could be used without Obsidian by manually using the Mix Tasks and a Markdown editor of your choice.\n\n> ##","ref":"magma-user-guide-introduction-to-magma-article-section.html"},{"type":"extras","title":"Info {: .info} - Introduction","doc":">\n> In the first version of Magma, only the OpenAI API is supported for automatic prompt execution. However, the system is designed with an LLM adapter facility that allows for the implementation of other LLMs in the future. Additionally, manual prompt execution is supported, which can be used to execute the prompts with anything you want.","ref":"magma-user-guide-introduction-to-magma-article-section.html#info-info"},{"type":"extras","title":"The Basic Idea - Introduction","doc":"The fundamental concept behind Magma is straightforward:\n\n- You, the user, collect project-relevant knowledge in \"[Atomic Notes](https://www.dsebastien.net/the-value-of-atomic-notes/)\" within an Obsidian vault. Atomic notes are individual units of knowledge that focus on one specific idea or concept. \n\n- Using [transclusions](https://en.wikipedia.org/wiki/Transclusion) — a concept borrowed from hypertext that allows for the insertion of the content of a document into another document — you can generate LLM prompts that contain the necessary knowledge for the LLM. Magma's transclusion resolution feature ensures that prompts consisting of transclusions are compiled into a proper prompt before execution.\n\n- In addition to user-defined custom prompts, Magma also defines a system of documents for a predefined workflow with predefined (but editable) prompts for the generation of various documentation artefacts, such as ModuleDocs, project readme, etc., including longer texts, as user guides for example.","ref":"magma-user-guide-introduction-to-magma-article-section.html#the-basic-idea"},{"type":"extras","title":"Installation and setup","doc":"<!-- ExDoc doesn't support YAML frontmatter\n\n---\nmagma_type: Artefact.Version\nmagma_artefact: Article\nmagma_concept: \"[[Magma User Guide - Installation and setup]]\"\nmagma_draft: \"[[Generated Magma User Guide - Installation and setup (article section) (2023-10-29T03:29:08)]]\"\ncreated_at: 2023-10-29 05:33:00\ntags: [magma-vault]\naliases: []\n---\n\n-->\n\n# Installation and setup\n\nThis section provides instructions on how to install and setup Magma within an Elixir project. \n\n> ##","ref":"magma-user-guide-installation-and-setup-article-section.html"},{"type":"extras","title":"Warning {: .warning} - Installation and setup","doc":">\n> Please note that Magma has only been tested with macOS so far. However, there is nothing that prevents it from running on other systems such as Windows and Linux. If you're using these, please read [this issue](https://github.com/marcelotto/magma/issues/1).\n\nAlthough primarily developed for Elixir projects, Magma can be useful in a variety of contexts. The following instructions assume you already have an Elixir project in which Magma will be installed. If you want to use Magma for general use cases, refer to the section, \"Installation for non-Elixir devs\" at the end.","ref":"magma-user-guide-installation-and-setup-article-section.html#warning-warning"},{"type":"extras","title":"Installation in an existing Elixir project - Installation and setup","doc":"Firstly, you need to set up the Magma Hex package as a development dependency in the `mix.exs` file of your project.\n\n``` elixir\ndef deps do\n  [\n    {:magma, \"~> 0.2\", only: [:dev, :test]}\n  ]\nend\n```\n\n> ##","ref":"magma-user-guide-installation-and-setup-article-section.html#installation-in-an-existing-elixir-project"},{"type":"extras","title":"Warning {: .warning} - Installation and setup","doc":">\n> If you're running on Apple Silicon you might experience problems with Rambo. You'll have to switch to the GitHub master version until the next version is released. See this issue: https://github.com/jayjun/rambo/pull/13#issuecomment-1193371511\n\nMagma relies on Pandoc, which needs to be installed separately. Make sure you have at least version 3.1.7. Refer to the [Pandoc installation guide](https://pandoc.org/installing.html) for more details.\n\nTo open the Magma Vault in Obsidian, you must have Obsidian is installed of course. We recommend using version 1.4 or above as links in the YAML frontmatter are properly supported from this version onward.\n\nFor best experience and control over the execution, Magma uses the OpenAI API. This requires adding the [Openai.ex](https://github.com/mgallo/openai.ex) package to your `mix.exs`.\n\n``` elixir\ndef deps do\n  [\n    {:magma, \"~> 0.1\", only: [:dev, :test]},\n    {:openai, \"~> 0.5\", only: [:dev]}\n  ]\nend\n```\n\nYou also need to set up your OpenAI API credentials in your `config.exs`. To avoid putting credentials in a file under version control, it is recommended to store them in environment variables as follows:\n\n``` elixir\nconfig :openai,  \n  api_key: {:system, \"OPENAI_API_KEY\"},  \n  organization_key: {:system, \"OPENAI_ORGANIZATION_KEY\"},  \n  http_options: [recv_timeout: 300_000]\n```\n\n> ##","ref":"magma-user-guide-installation-and-setup-article-section.html#warning-warning"},{"type":"extras","title":"Info {: .info} - Installation and setup","doc":">\n> The default HTTP timeout is increased here, which is strongly recommended as Magma prompts can become quite large, resulting in lengthy executions especially with the GPT-4 model. For more details on the configuration options of Openai.ex refer to its README.\n\nYou can set these environment variables in an `.envrc` file in your project directory:\n\n```sh\n# find it at https://platform.openai.com/account/api-keys  \nexport OPENAI_API_KEY=your-api-key\n# find it at https://platform.openai.com/account/org-settings under \"Organization ID\"  \nexport OPENAI_ORGANIZATION_KEY=your-org-key\n```","ref":"magma-user-guide-installation-and-setup-article-section.html#info-info"},{"type":"extras","title":"Magma vault creation - Installation and setup","doc":"Create a new Magma vault for your project with the `Mix.Tasks.Magma.Vault.Init` Mix task:\n\n``` sh\n$ mix magma.vault.init \"Name of your project\" [BaseVaultName]\n```\n\nYou must specify a project name as the first mandatory parameter. As an optional second argument, you can specify the name of a BaseVault. A BaseVault is an Obsidian vault preconfigured with Obsidian themes and plugins. You can also specify the local path to a self-defined BaseVault. If no BaseVault is specified, the default BaseVault is used.\n\nBy default, a Magma vault is stored under the `docs.magma` directory within your project. This can be changed by configuring the `dir` application key of the `magma` app in `config.exs`:\n\n``` elixir\nconfig :magma,  \n  dir: \"your_magma_vault/\"\n```\n\n> ##","ref":"magma-user-guide-installation-and-setup-article-section.html#magma-vault-creation"},{"type":"extras","title":"Warning {: .warning} - Installation and setup","doc":">\n> At the current state of the project, you can only change the name of directory here and not specify a completely separate directory outside the Elixir project. This is not supported yet.\n\nYou can also configure a set of tags to be added on all generated documents with the `default_tags` key in your `config.exs`:\n\n```elixir\nconfig :magma,  \n  default_tags: [\"magma-vault\"]\n```\n\n> ##","ref":"magma-user-guide-installation-and-setup-article-section.html#warning-warning"},{"type":"extras","title":"Info {: .info} - Installation and setup","doc":">\n> While Magma is primarily configured using specific config documents in the `magma.config` subdirectory of the Magma vault, including the `default_tags` through the `Magma.system.config` file, there are certain configuration options that should be set up in the application config. This ensures that the respective defaults are provided for the vault config files that are created during vault initialization. This is especially important when these settings are already utilized during vault initialization, such as the `default_tags` setting. It is used for the creation of all documents during vault initialization.","ref":"magma-user-guide-installation-and-setup-article-section.html#info-info"},{"type":"extras","title":"Code sync - Installation and setup","doc":"If you want to add documents for modules that were created after the initial vault creation, you can do so with the `Mix.Tasks.Magma.Vault.Sync.Code` Mix task:\n\n``` sh\n$ mix magma.vault.sync.code\n```\n\nA code sync creates corresponding documents for the generation of Magma artefacts for all public and non-ignored modules. A module is ignored if it has a `# Magma pragma: ignore` comment at the beginning of its source code, or if it is marked as hidden (e.g. with `@moduledoc false`) and does not have a `# Magma pragma: include` comment at the beginning of its source code file.","ref":"magma-user-guide-installation-and-setup-article-section.html#code-sync"},{"type":"extras","title":"Installation for non-Elixir devs - Installation and setup","doc":"Firstly, install Erlang and Elixir following [this guide](https://elixir-lang.org/install.html).\n\nThen navigate to the directory where you want to create the Magma vault with your Markdown files and run the following command with the name of the directory and change into this directory.\n\n``` sh\n$ mix new my_magma_vault\n$ cd my_magma_vault\n```\n\nNow, you can continue with the \"Installation in an existing Elixir project\" above. Please note:\n\n- After editing the deps in the `mix.exs` file, you need to fetch the specified dependencies: \n``` sh\n$ mix deps.get\n```\n- Since you're not interested in documenting code, add the `--no-code-sync` option during vault initialization: \n``` sh\n$ mix magma.vault.init \"Name of your project\" --no-code-sync\n```","ref":"magma-user-guide-installation-and-setup-article-section.html#installation-for-non-elixir-devs"},{"type":"extras","title":"Transclusion Resolution","doc":"<!-- ExDoc doesn't support YAML frontmatter\n\n---\nmagma_type: Artefact.Version\nmagma_artefact: Article\nmagma_concept: \"[[Magma User Guide - Transclusion Resolution]]\"\nmagma_draft: \"[[Generated Magma User Guide - Transclusion Resolution (article section) (2023-10-29T22:32:37)]]\"\ncreated_at: 2023-10-29 22:38:47\ntags: [magma-vault]\naliases: []\n---\n\n-->\n\n# Transclusion Resolution\n\nTransclusion is a fundamental concept in Magma, playing a crucial role in the generation of prompts for LLMs. This section aims to provide an understanding of transclusions, atomic notes, and the process of transclusion resolution in Magma.","ref":"magma-user-guide-transclusion-resolution-article-section.html"},{"type":"extras","title":"What are Transclusions? - Transclusion Resolution","doc":"[Transclusions](https://en.wikipedia.org/wiki/Transclusion) refer to the inclusion of a document or part of a document into another document by reference. Instead of directly copying the content, a directive or a piece of code is inserted, instructing the system to fetch and embed the original content. This method ensures that the content is maintained in a single place and displayed in multiple locations as needed. \n\nThe main advantages of transclusions are:\n\n1.  **Dynamic Content Updating**: Changes made to the original content are automatically reflected wherever the content has been transcluded, ensuring consistency.\n2.  **Efficiency**: Redundancy is reduced and storage space is saved as the same content doesn't need to be stored in multiple places.\n3.  **Content Management**: Single source of truth for content that might need to be displayed in multiple locations simplifies content management.\n\nIn Obsidian, transclusions are written as `![[Some Document]]` or section transclusions  as `![[Some Document#Some Section]]`.","ref":"magma-user-guide-transclusion-resolution-article-section.html#what-are-transclusions"},{"type":"extras","title":"Importance of Atomic Notes - Transclusion Resolution","doc":"Atomic notes are small, self-contained notes that capture a single idea, topic, or piece of information. They are indispensable in systems where transclusions are used extensively, such as Magma. Here's why:\n\n- **Reusability**: Atomic notes can be easily transcluded in various contexts without carrying over irrelevant information.\n- **Maintainability**: Updates made in one place ensure that all transclusions of the note are up-to-date, thus maintaining consistency across the entire system.\n- **Composability**: Atomic notes can be combined to construct more complex ideas or documents, allowing for flexible content creation.\n- **Clarity**: Each atomic note serves a specific purpose and addresses a specific point, enhancing the overall clarity of the content system.\n\nSince Obsidian (and Magma) also support transclusions of section, atomic notes not necessarily need to be stored all in separate documents, but can be grouped as sections in a document, much like files in folders, when it is useful.","ref":"magma-user-guide-transclusion-resolution-article-section.html#importance-of-atomic-notes"},{"type":"extras","title":"Transclusion Resolution in Magma - Transclusion Resolution","doc":"Transclusion resolution in Magma refers to the process of resolving an Obsidian transclusion by replacing it with the referenced content. This is crucial for the composition of LLM prompts, which are defined as compositions of transclusions that must be resolved before the LLM execution, since the LLM can't access the referenced content. The content of the referenced document or document section is not inserted unchanged, however. Instead, it undergoes the following processing steps:\n\n- Comments (`<!-- comment -->`) are removed.\n- Internal links are replaced with the target (or their display text when available) as plain text. (However, the style in which internal links are to be resolved can be customized via the `link_resolution_style` property of the `Magma.system.config` document. Permissible values there are: `plain` (default), `emph`, `strong`, `underline`.)\n- Transclusions within the transcluded content itself are resolved recursively (unless it would result in an infinite recursion)\n- If the transcluded content (after removing the comments), consists exclusively of a heading with no content below it, the transclusion is resolved with the empty string.\n- The level of the transcluded sections is adjusted according to the current level at the point of the transclusion.\n\nThere are three kinds of transclusions in Magma which are resolved slightly differently:\n\n1. *Inline transclusions*: remove the leading header\n2. *Custom header transclusions*: replace the leading header\n3. *Empty header transclusions*: keep the leading header\n\nAnother difference between these transclusion resolution types is how they handle the prologue, i.e., text before the document title, on complete document transclusion resolutions. Both kinds of header transclusions omit prologue, while inline transclusions keep the prologue.\n\n> ##","ref":"magma-user-guide-transclusion-resolution-article-section.html#transclusion-resolution-in-magma"},{"type":"extras","title":"Warning {: .warning} - Transclusion Resolution","doc":">\n> Unfortunately, the different types of transclusion are not visible in Obsidian, as the first heading is always displayed there. This can become confusing, especially with nested transclusions.\n\nTo better understand the differences between transclusion types, let's consider an example document:\n\n``` markdown\nSome text before the document title like this, is called prologue in Magma.\n\n# Some document\n\nId occaecat fugiat ea anim adipiscing.","ref":"magma-user-guide-transclusion-resolution-article-section.html#warning-warning"},{"type":"extras","title":"Some Section - Transclusion Resolution","doc":"Aliqua ea reprehenderit aliquip aliquip laborum.\n```\n\n1.  *Inline transclusions* stand alone in their own paragraph:\n\n``` markdown\nDolor ad eiusmod, eu ea.\n\n![[Some Document#Some Section]]\n\nCulpa duis, ut id excepteur.\n```\n\nThis will be resolved to this result:\n\n``` markdown\nDolor ad eiusmod, eu ea.\n\nAliqua ea reprehenderit aliquip aliquip laborum.\n\nCulpa duis, ut id excepteur.\n```\n\n2.  *Custom header transclusions* are placed at the end of a header. The removed header is replaced with the one in this header.\n\n``` markdown\nDolor ad eiusmod, eu ea.\n\n#","ref":"magma-user-guide-transclusion-resolution-article-section.html#some-section"},{"type":"extras","title":"Custom section title ![[Some Document#Some Section]] - Transclusion Resolution","doc":"Culpa duis, ut id excepteur.\n```\n\nThis will be resolved to this result:\n\n``` markdown\nDolor ad eiusmod, eu ea.\n\n#","ref":"magma-user-guide-transclusion-resolution-article-section.html#custom-section-title-some-document-some-section"},{"type":"extras","title":"Custom section title - Transclusion Resolution","doc":"Aliqua ea reprehenderit aliquip aliquip laborum.\n\nCulpa duis, ut id excepteur.\n```\n\n3.  *Empty header transclusions* keep the original header of the transcluded content. They are written in a header like the custom header transclusion, but define no custom header title.\n\n``` markdown\nDolor ad eiusmod, eu ea.\n\n### ![[Some Document#Some Section]]\n\nCulpa duis, ut id excepteur.\n```\n\nThis will be resolved to this result:\n\n``` markdown\nDolor ad eiusmod, eu ea.\n\n#","ref":"magma-user-guide-transclusion-resolution-article-section.html#custom-section-title"},{"type":"extras","title":"Some Section - Transclusion Resolution","doc":"Aliqua ea reprehenderit aliquip aliquip laborum.\n\nCulpa duis, ut id excepteur.\n```\n\n> ##","ref":"magma-user-guide-transclusion-resolution-article-section.html#some-section"},{"type":"extras","title":"Warning {: .warning} - Transclusion Resolution","doc":">\n> Intra-document transclusions, i.e., transclusions of sections inside the same document, are currently not supported due to a too coarse transclusion recursion detection. However, support for such transclusions is planned for the next version.","ref":"magma-user-guide-transclusion-resolution-article-section.html#warning-warning"},{"type":"extras","title":"Custom Prompts and Prompt Execution","doc":"<!-- ExDoc doesn't support YAML frontmatter\n\n---\nmagma_type: Artefact.Version\nmagma_artefact: Article\nmagma_concept: \"[[Magma User Guide - Custom Prompts and Prompt Execution]]\"\nmagma_draft: \"[[Generated Magma User Guide - Custom Prompts and Prompt Execution (article section) (2023-10-30T17:50:59)]]\"\ncreated_at: 2023-10-30 19:35:54\ntags: [magma-vault]\naliases: []\n---\n\n-->\n\n\n# Custom Prompts and Prompt Execution\n\nApart from the predefined prompts for the generation of specific artefacts discussed in the following sections, Magma also provides a feature to create and execute custom prompts. We start with them to introduce prompt execution, which also applies to the more involved artefact prompts.","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html"},{"type":"extras","title":"Creating a Custom Prompt - Custom Prompts and Prompt Execution","doc":"In Magma, you can create a custom prompt either via the Mix task `Mix.Tasks.Magma.Prompt.Gen`:\n\n``` sh\n$ mix magma.prompt.gen \"Name of prompt\"\n```\n\nor from within Obsidian using the command palette or the Cmd-Ctrl-P hotkey for the QuickAdd command \"Custom Magma prompt\". This triggers the same Mix task via the Obsidian ShellCommand plugin.\n\n> ##","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#creating-a-custom-prompt"},{"type":"extras","title":"Warning {: .warning} - Custom Prompts and Prompt Execution","doc":">\n> Just like the name of any Obsidian document, the name of the prompt document must be unique. A good practice is to stick to a common naming scheme for prompts, e.g., \"Prompt for ...\", to ensure that the prompt documents never conflict with non-prompt documents.\n\nThis process creates a Magma prompt document, a special type of Magma document. It is saved in the `custom_prompt/` subdirectory of the Magma vault. \n\n> ##","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#warning-warning"},{"type":"extras","title":"Magma documents {: .neutral} - Custom Prompts and Prompt Execution","doc":"> \n> Magma documents are Markdown files with a particular structure and semantic rules specific to Magma. \n\nBelow is an example of a newly created custom prompt named \"Example prompt\":\n\n``` markdown\n---\nmagma_type: Prompt\nmagma_generation_type: OpenAI\nmagma_generation_params: {\"model\":\"gpt-4\",\"temperature\":0.6}\ncreated_at: 2023-12-04T16:32:39\ntags: [magma-vault]\naliases: []\n---\n\n**Generated results**\n\n``dataview\nTABLE\n\ttags AS Tags,\n\tmagma_generation_type AS Generator,\n\tmagma_generation_params AS Params\nWHERE magma_prompt = [[]]\n``\n\n**Actions**\n\n``button\nname Execute\ntype command\naction Shell commands: Execute: magma.prompt.exec\ncolor blue\n``\n``button\nname Execute manually\ntype command\naction Shell commands: Execute: magma.prompt.exec-manual\ncolor blue\n``\n``button\nname Copy to clipboard\ntype command\naction Shell commands: Execute: magma.prompt.copy\ncolor default\n``\n\n# Example prompt","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#magma-documents-neutral"},{"type":"extras","title":"System prompt - Custom Prompts and Prompt Execution","doc":"![[Magma.system.config#Persona|]]\n\n#","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#system-prompt"},{"type":"extras","title":"Context knowledge - Custom Prompts and Prompt Execution","doc":"The following sections contain background knowledge you need to be aware of, but which should NOT necessarily be covered in your response as it is documented elsewhere. Only mention absolutely necessary facts from it. Use a reference to the source if necessary.\n\n![[Magma.system.config#Context knowledge|]]\n\n##","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#context-knowledge"},{"type":"extras","title":"Description of the Magma project ![[Project#Description|]] - Custom Prompts and Prompt Execution","doc":"","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#description-of-the-magma-project-project-description"},{"type":"extras","title":"Request - Custom Prompts and Prompt Execution","doc":"```\n\n> ##","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#request"},{"type":"extras","title":"Warning {: .warning} - Custom Prompts and Prompt Execution","doc":">\n> To prevent problems with the Markdown processor, code blocks with three backticks are written in this guide using two backticks to maintain proper rendering. In the actual documents the code blocks are written correctly with three backticks.\n\nThe custom prompt document consists of several parts:\n\n-   The YAML frontmatter includes several Magma-specific properties:\n    -   `magma_type`: the Magma document type\n    -   `magma_generation_type` and `magma_generation_params`: prompt-specific parameters for configuring the prompt execution\n    -   `created_at`, `tags`, and `aliases`: standard Obsidian properties\n-   The prologue (the text before the initial document title header) contains some document controls which require Obsidian to get rendered:\n    -   A DataView table showing a list of the generated prompt results\n    -   A series of buttons for different actions\n-   The main body of the prompt contains two sections:\n    -   \"System prompt\": This section becomes the system prompt of the OpenAI API request. It includes the persona and a \"Context knowledge\" subsection for providing background knowledge to the LLM.\n    -   \"Request\": This section is where you write the actual prompt.","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#warning-warning"},{"type":"extras","title":"Prompt configuration - Custom Prompts and Prompt Execution","doc":"As you can see from the example prompt above, this contains some transclusions of sections from documents with a `.config` extension. These documents, which are located in the `magma.config` subdirectory of the vault, contain some basic setting options via properties in the YAML frontmatter as well as some sections that are transcluded or copied in various places. Of the numerous configuration files, however, only the `Magma.system.config` document, with the most general settings that are relevant for all prompts, plays a role for custom prompts. On the one hand, the persona, i.e. the text that opens the system prompt part in all prompts, can be defined there. On the other hand, parts of the context knowledge that should appear in all prompts can be defined here.\n\n> ##","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#prompt-configuration"},{"type":"extras","title":"Context knowledge {: .neutral} - Custom Prompts and Prompt Execution","doc":"> \n> In Magma, \"context knowledge\" refers to knowledge that should not necessarily be part of the result of a prompt, but is relevant to its understanding or execution. Typically, these are transclusions of various atomic notes.\n\n> ##","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#context-knowledge-neutral"},{"type":"extras","title":"Info {: .info} - Custom Prompts and Prompt Execution","doc":"> \n> Note that the project description is always part of the context knowledge of every prompt, except when it comes to prompts that have the project itself as its subject, where this description is included in a more prominent manner. Due to this conditional transclusion, the project description is not part of the context knowledge section of the `Magma.system.config` document.\n\nThe initial content of custom prompts can be customized with the Obsidian template in the directory `templates/custom_prompt.md` of the Magma vault. However, the basic structure of a \"System prompt\" and a \"Request\" section should remain unchanged.","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#info-info"},{"type":"extras","title":"Writing a Custom Prompt - Custom Prompts and Prompt Execution","doc":"1. Unless you're working on a popular project the LLM has enough knowledge from its training, you should provide the basics of your project. So, ensure that a project description is provided in the \"Description\" section of the \"Project\" concept document. \n2.  Write your request in the \"Request\" section.\n3.  Add more necessary background knowledge that the LLM needs to understand your request to the \"Context knowledge\" section. This is ideally done via transclusion of atomic notes.","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#writing-a-custom-prompt"},{"type":"extras","title":"Executing the Prompt - Custom Prompts and Prompt Execution","doc":"Magma provides two ways to execute the prompt: automatic execution and manual execution. In both cases, the prompt result is saved in a separate prompt result document named after the original prompt with a timestamp. Like any prompt result it is placed in a subdirectory `__prompt_results__` of the directory where the prompt document is stored.\n\n> ##","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#executing-the-prompt"},{"type":"extras","title":"Info {: .info} - Custom Prompts and Prompt Execution","doc":"> \n> The Magma vault directory contains its own `.gitignore` file in which `__prompt_results__/` is listed by default, so they won't be version controlled.\n\n#","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#info-info"},{"type":"extras","title":"Manual Execution - Custom Prompts and Prompt Execution","doc":"Manual execution can be triggered from within Obsidian via the \"Execute manually\" button. This creates an empty prompt result document, which should show up in the \"Generated results\" table in the prologue and copies the compiled prompt with all transclusions resolved to the clipboard. The prompt from the clipboard can then be copied to the chatbot of your choice (ChatGPT, Claude, Bard etc.), executed there and the result can be copied to the respective result document.\n\nWhen executing manually via the `Mix.Tasks.Magma.Prompt.Exec` Mix task directly:\n\n``` sh\n$ mix magma.prompt.exec \"Name of prompt\" --manual\n```\n\nyou are prompted on the shell to paste back the result, which is then added to the created prompt result document automatically. \n\nIf you just want to execute the prompt and not save the result back into your vault, you can use the `Copy` button or the `Mix.Tasks.Magma.Prompt.Copy` Mix task:\n\n``` sh\n$ mix magma.prompt.copy \"Name of prompt\"\n```\n\n#","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#manual-execution"},{"type":"extras","title":"Automatic Execution - Custom Prompts and Prompt Execution","doc":"Automatic execution is triggered via the \"Execute\" button or the `Mix.Tasks.Magma.Prompt.Exec` Mix task (without the `--manual` option).\n\nIn automatic execution, the `magma_generation_type` and `magma_generation_params` properties of the YAML frontmatter determine how the prompt is executed. The `magma_generation_type` determines which implementation of an LLM adapter (`Magma.Generation`) should be used. Currently, only the OpenAI API implementation (`Magma.Generation.OpenAI`) is available. The `magma_generation_params` set the values for the parameters of the selected adapter.\n\n> ##","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#automatic-execution"},{"type":"extras","title":"Warning {: .warning} - Custom Prompts and Prompt Execution","doc":">\n> Unfortunately, the property editor in Obsidian does not currently support editing JSON parameters. Therefore, you need to switch to source mode to edit `magma_generation_params` in Obsidian.\n\nThe prompt is then executed using the configured LLM adapter and its parameters, and the result is stored in a prompt result document upon completion. Execution can take several minutes, especially with GPT-4, which is highly recommended for its superior results. Completion is signaled by an Obsidian notification. If you are not satisfied with a prompt result, you can delete it using the \"Delete\" button in the prologue and try again with different parameters.\n\nYou can configure the default values for the `magma_generation_type` and `magma_generation_params` properties via the respective YAML frontmatter properties `Magma.system.config` document. The `default_generation_type` key sets the `Magma.Generation` adapter to be used for new prompt documents, while the configuration for the adapter sets the default values for the `magma_generation_params` property of new prompt documents. Note, that you can still adapt them there individually.","ref":"magma-user-guide-custom-prompts-and-prompt-execution-article-section.html#warning-warning"},{"type":"extras","title":"Creating and Understanding Magma Artefacts","doc":"<!-- ExDoc doesn't support YAML frontmatter\n\n---\nmagma_type: Artefact.Version\nmagma_artefact: Article\nmagma_concept: \"[[Magma User Guide - Creating and Understanding Magma Artefacts]]\"\nmagma_draft: \"[[Generated Magma User Guide - Creating and Understanding Magma Artefacts (article section) (2023-10-31T02:26:01)]]\"\ncreated_at: 2023-10-31 02:26:17\ntags: [magma-vault]\naliases: []\n---\n\n-->\n\n# Creating and Understanding Magma Artefacts\n\nIn Magma, we have the ability to generate what we call *artefacts*. These are outputs or products of the Magma environment that are created using predefined workflows and prompts. The basis for this is the Magma artefact model, which we'll introduce in this section. \n\nMagma artefacts are things we want to generate. For example, these could be documentation artefacts like moduledocs for the API documentation, user guides, cheatsheets, a project website or README. There could also be code artefacts like test factories, properties etc., although these are not yet supported in the current version of Magma.","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html"},{"type":"extras","title":"Magma Artefact Model - Creating and Understanding Magma Artefacts","doc":"The Magma artefact model is based on the concept of *Matters* and *Artefacts*. A Magma artefact is always about a specific subject matter, which is represented as an instance of a Magma matter type. Such a Magma matter instance is described in a concept document (`Magma.Concept`). The concept document also includes sections with prompts for the different kinds of artefacts for that matter. \n\nIn the next step of the artefact generation process, the artefact prompt document (`Magma.Artefact.Prompt`) is composed. This special kind of Magma prompt document has the goal of generating a concrete version of an artefact in a Magma artefact version document (`Magma.Artefact.Version`). The content structure is determined by the artefact type and is filled with the relevant parts of the concept document and eventually some matter-specific parts, for example, the code of the module in the case of `Magma.Matter.Module` matter type.\n\nAfter the execution of the artefact prompt, the best prompt result is selected as a draft for the artefact version document, which is finally edited and finalized by the user. \n\nHere's an illustrative sequence diagram showing the connection between these elements:\n\n``` mermaid\nsequenceDiagram\n    participant Ma as Magma.Artefact\n    participant Mm as Magma.Matter\n    participant Mc as Magma.Concept\n    participant Map as Magma.Artefact.Prompt\n    participant Mpr as Magma.PromptResult\n    participant Mav as Magma.Artefact.Version\n\n    Ma->>Mm: is about\n    Mm->>Mc: is described by\n    Ma->>Map: determines structure of\n    Mc->>Map: provides content for\n    Map->>Mpr: is executed to generate\n    Mpr->>Mav: is selected draft for\n    Mav->>Ma: realizes\n```\n\n\nIn summary, the general process of creating a Magma artefact involves the following steps:\n\n1. **Write the concept**: Provide a description of the (subject) matter in the \"Description\" section of the concept document. Add necessary background knowledge in the \"Context knowledge\" section that helps to understand the description or generate artefacts about this matter. Customize the default prompt of the artefact to be created in the \"Artefacts\" section.\n2. **Review the prompt** and refine the concept if necessary.\n3. **Execute the prompt**. This can be done multiple times until you're satisfied with the result. Refine the concept and/or adapt the generation parameters in this iterative process.\n4. **Select the best prompt result** as a draft for the final version.\n5. **Edit and finalize** the final artefact version.\n\nLet's see this workflow in action, by demonstrating the generation of the project README and a moduledoc.","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#magma-artefact-model"},{"type":"extras","title":"Generating a Project README - Creating and Understanding Magma Artefacts","doc":"The first step is to write the concept of the project in the concept document. This document was already created during the vault initialization along with a prompt for the README artefact.\n\n#","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#generating-a-project-readme"},{"type":"extras","title":"The Concept Document - Creating and Understanding Magma Artefacts","doc":"```markdown\n---\nmagma_type: Concept\nmagma_matter_type: Project\nmagma_matter_name: Example\ncreated_at: 2023-10-06 16:03:10\ntags: [magma-vault]\naliases: [Example project, Example-project]\n---\n# Example project","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#the-concept-document"},{"type":"extras","title":"Description - Creating and Understanding Magma Artefacts","doc":"<!--\nWhat is the Example project about?\n-->\n\n\n# Context knowledge\n\n<!--\nThis section should include background knowledge needed for the model to create a proper response, i.e. information it does not know either because of the knowledge cut-off date or unpublished knowledge.\n\nWrite it down right here in a subsection or use a transclusion. If applicable, specify source information that the model can use to generate a reference in the response.\n-->\n\n\n\n\n# Artefacts","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#description"},{"type":"extras","title":"README - Creating and Understanding Magma Artefacts","doc":"- Prompt: [[Prompt for project README]]\n- Final version: [[README]]\n\n#","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#readme"},{"type":"extras","title":"Readme prompt task - Creating and Understanding Magma Artefacts","doc":"Generate a README for project 'Example' according to its description and the following information:  \n  \nHex package name: app_name\nRepo URL: https://github.com/github_username/repo_name  \nDocumentation URL: https://hexdocs.pm/app_name/\nHomepage URL:  \nDemo URL:  \nLogo path: logo.jpg  \nScreenshot path:  \nLicense: MIT License  \nContact: Your Name - [@twitter_handle](https://twitter.com/twitter_handle) - your@email.com  \nAcknowledgments:  \n  \n(\"n/a\" means not applicable and should result in a removal of the respective parts)\n\n```\n\nThe concept document has a YAML frontmatter which includes Obsidian properties (`created_at`, `tags`, `aliases`), the `magma_type` property specifying the Magma document type, and two new properties:\n\n- `magma_matter_type`: Specifies the Magma matter type, in this case, `Project`.\n- `magma_matter_name`: Specifies the name of the concrete matter, in this case, the name of the project. For matter types, where the matter name is the same as the document name, this property is not needed.\n\nThe main body of the concept includes:\n\n##","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#readme-prompt-task"},{"type":"extras","title":"a) Description of the Matter - Creating and Understanding Magma Artefacts","doc":"The \"Description\" section of any concept document is crucial as it provides a description of its subject matter, in this case, the project, which will be transcluded as the central part in the request part of the prompt. This description can be written directly into this section or transcluded from other documents. \n\n> ##","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#a-description-of-the-matter"},{"type":"extras","title":"Info {: .info} - Creating and Understanding Magma Artefacts","doc":">\n> As we saw in the last chapter, the project description is very important, as it is transcluded in the \"Context knowledge\" section of every custom prompt and every artefact prompt (except for those about the project itself, where it is transcluded more prominently since it's not just context knowledge in this case).\n\n##","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#info-info"},{"type":"extras","title":"b) Context Knowledge - Creating and Understanding Magma Artefacts","doc":"The \"Context knowledge\" section provides background information that helps to understand the matter and its description. For our project, for example, we could describe its ecosystem or some used technologies.\n\nWe encountered this section already in the introduction of [Custom prompts](Magma User Guide - Custom Prompts and Prompt Execution (article section).md). However, while in the case of a Custom prompt its content was specified directly in the prompt document, it is now specified in the concept document and transcluded in the prompts of all artefacts of this matter instance.\n\n##","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#b-context-knowledge"},{"type":"extras","title":"c) Artefacts - Creating and Understanding Magma Artefacts","doc":"The \"Artefacts\" section includes subsections for all supported artefact types of the respective matter type. These subsections contain links to the respective artefact prompt and artefact version document, and a \"Prompt task\" section with the default prompt for the respective artefact type, which can be customized or extended for this particular instance here in the concept document. \n\n> ##","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#c-artefacts"},{"type":"extras","title":"Tip {: .tip} - Creating and Understanding Magma Artefacts","doc":"> \n> If you want to customize the task prompt for an artefact type in general, you can do so in the \"Task prompt\" section of the config document of the respective artefact type in the `magma.config/artefacts` subdirectory of the vault. Note that this section is special because it is not transcluded, but interpreted as an EEx template that is evaluated when a concept document is created. The result of this evaluation is used as the content of the \"Prompt task\" section of the corresponding artefact type.\n\nIn the case of our project concept document, the only available artefact type in this first version of Magma is the README artefact. The system prompt for its generation is based on a template, which relies on some information, which should be provided in the given form in this section. If some fields do not apply for your project, you should write `n/a`.\n\n\n#","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#tip-tip"},{"type":"extras","title":"The Prompt Document - Creating and Understanding Magma Artefacts","doc":"After filling the concept document, the artefact prompt document should be reviewed with the transcluded content. Let's look at the artefact prompt document that was generated during the vault initialization.\n\n```markdown\n---\nmagma_type: Artefact.Prompt\nmagma_artefact: Readme\nmagma_concept: \"[[Project]]\"\nmagma_generation_type: OpenAI\nmagma_generation_params: {\"model\":\"gpt-4\",\"temperature\":0.6}\ncreated_at: 2023-10-24 16:05:35\ntags: [magma-vault]\naliases: []\n---\n\n**Generated results**\n\n``dataview\nTABLE\n\ttags AS Tags,\n\tmagma_generation_type AS Generator,\n\tmagma_generation_params AS Params\nWHERE magma_prompt = [[]]\n``\n\nFinal version: [[README]]\n\n**Actions**\n\n``button\nname Execute\ntype command\naction Shell commands: Execute: magma.prompt.exec\ncolor blue\n``\n``button\nname Execute manually\ntype command\naction Shell commands: Execute: magma.prompt.exec-manual\ncolor blue\n``\n``button\nname Copy to clipboard\ntype command\naction Shell commands: Execute: magma.prompt.copy\ncolor default\n``\n``button\nname Update\ntype command\naction Shell commands: Execute: magma.prompt.update\ncolor default\n``\n\n# Prompt for README","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#the-prompt-document"},{"type":"extras","title":"System prompt - Creating and Understanding Magma Artefacts","doc":"![[Magma.system.config#Persona|]]\n\n![[Readme.artefact.config#System prompt|]]\n\n#","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#system-prompt"},{"type":"extras","title":"Context knowledge - Creating and Understanding Magma Artefacts","doc":"The following sections contain background knowledge you need to be aware of, but which should NOT necessarily be covered in your response as it is documented elsewhere. Only mention absolutely necessary facts from it. Use a reference to the source if necessary.\n\n![[Magma.system.config#Context knowledge|]]\n\n![[Project.matter.config#Context knowledge|]]\n\n![[Readme.artefact.config#Context knowledge|]]\n\n![[Project#Context knowledge|]]","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#context-knowledge"},{"type":"extras","title":"Request - Creating and Understanding Magma Artefacts","doc":"![[Project#Readme prompt task|]]\n\n#","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#request"},{"type":"extras","title":"Description of the 'Example' project ![[Project#Description|]] - Creating and Understanding Magma Artefacts","doc":"```\n\nYou should notice its structure is very similar to the prompt shown in the previous page about \"Custom prompts\", but has some notable differences:\n\n- In the YAML frontmatter, the properties `magma_artefact` and `magma_concept` have been added, which specify the artefact type and link to the concept document. \n- The **Generated results** now contains a dedicated link to the final artefact version document. \n- The **Actions** buttons now also have an update button, which you can use to regenerate the prompt. Although transcluded content is displayed automatically, it is sometimes necessary to regenerate the prompt (for example when source code, which is included for some artefact prompts, was modified, like for moduledocs), which can be done with this button or the `Mix.Tasks.Magma.Prompt.Update` Mix task.\n- The \"System prompt\" section transcludes a section of the respective artefact type config document, which can be adapted to your specific needs. In the case of the README artefact type it contains a template for the README to be generated, which uses the fields of the form in the README artefact section in the concept document seen above.\n- The \"Context knowledge\" section now includes a lot more transclusions from additional config documents, which allows you to compose the necessary knowledge more granular. After the \"Context knowledge\" section from the general `Magma.system.config` document that is always transcluded, we now see the following additional \"Context knowledge\" section transclusions:\n\t- from a config document for the matter type of the concept, in this case of the `Project` subject matter, the `Project.matter.config` document,\n\t- from a config document for the artefact type, in this case of a `Readme` artefact, the `Readme.artefact.config` document,\n\t- and finally, the \"Context knowledge\" section from the concept document above.\n\nThe artefact prompt can be executed in the same way as described in the \"Custom Prompts and Prompt Execution\" section of this guide. You may need to execute the prompt multiple times until you're satisfied with the result.\n\n#","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#description-of-the-example-project-project-description"},{"type":"extras","title":"The Prompt Result Documents - Creating and Understanding Magma Artefacts","doc":"The prompt results now have an additional \"Select as draft version\" button, which you can use to select the best prompt result as a template for the artefact version. \n\nAlternatively, you can use the `Mix.Tasks.Magma.Artefact.SelectDraft` Mix task directly:\n\n```sh\n$ mix magma.artefact.select_draft \"Name of prompt result\"\n```\n\n> ##","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#the-prompt-result-documents"},{"type":"extras","title":"Warning {: .warning} - Creating and Understanding Magma Artefacts","doc":">\n> If a README already exists and must be overwritten, you must use the Mix task to confirm the overwrite. If you want to use the button, you must manually remove the old version beforehand, as a confirmation is currently not supported in Obsidian.\n\n#","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#warning-warning"},{"type":"extras","title":"The Artefact Version Document - Creating and Understanding Magma Artefacts","doc":"After selecting the best prompt result as a draft, a `README.md` file is created in the project's root directory and filled with the result. A symbolic link to this file is then created in the Magma vault where the artefact version is normally stored (`artefacts/final/project/README/README.md`). This allows the README to be opened and edited in Obsidian. \n\nAs can be seen in this case of a README, artefact version documents are not always proper Magma documents, in the sense that they are properly typed with a `magma_type`, since this isn't possible, when the system consuming the artefact doesn't support YAML frontmatter.\n\nWith that, you can now complete the final version of the artefact.","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#the-artefact-version-document"},{"type":"extras","title":"Generating module documentation for the API documentation - Creating and Understanding Magma Artefacts","doc":"The process of generating the ModuleDoc for API documentation is quite similar to that of generating a README. The related concept and artefact prompts are created either during the initialization of the vault or by a subsequent code sync (refer to the [Installation and setup](Magma User Guide - Installation and setup (article section).md) page for details). \n\nAlthough we're dealing with another matter type here, the concept documents are quite akin to those for the project which we saw in previous README example. The primary differences are:\n\n- The `magma_matter_type` is `Module` instead of `Project`.\n- The hints for the content to be written are tailored according to the `Module` matter type.\n- The artefacts for the `Module` matter type are different. In the current version of Magma, only the `ModuleDoc` artefact type is available.\n\nThe primary goal of the `ModuleDoc` artefact is to generate the content for the `@moduledoc` string in the module's code. In fact, when the final artefact version is generated, you can use the `Magma` module as a replacement for the `@moduledoc` definition:\n\n``` elixir\ndefmodule Some.Module do\n  use Magma\n  # will be replaced with a @moduledoc  \n\n  # some code   \nend\n```\n\n> ##","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#generating-module-documentation-for-the-api-documentation"},{"type":"extras","title":"Warning {: .warning} - Creating and Understanding Magma Artefacts","doc":">\n> If you decide to include your moduledocs with `use Magma`, be aware that if you're writing a library and your users should be able to use these docs on their machines, e.g. with the `h` helper in IEx you'll have to include the Magma documents with the final moduledocs in your package like this:\n> \n> ```elixir\n> defp package do  \n>   [  \n>     # ...\n>     files:  ~w[lib priv mix.exs docs.magma/artefacts/final/modules/**/*.md]\n>   ]  \n> end  \n> ```\n\nHowever, the artefact prompt also asks for function docs for two reasons:\n\n1.  It's challenging to get the language model to ignore the functions and focus solely on the `@moduledoc`. It tends to describe the functions present in the shown implementation.\n2.  Even though Magma doesn't currently offer a similar integration for function docs as it does for `@moduledoc`, they are useful as copy-paste templates.\n\nHere is a detailed look at the prompt for the `ModuleDoc` artefact (without the YAML frontmatter and prologue with the document controls, which are similar to the README prompt above):\n\n```markdown\n# Prompt for ModuleDoc of Some.Example","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#warning-warning"},{"type":"extras","title":"System prompt - Creating and Understanding Magma Artefacts","doc":"![[Magma.system.config#Persona|]]\n\n![[ModuleDoc.artefact.config#System prompt|]]\n\n#","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#system-prompt"},{"type":"extras","title":"Context knowledge - Creating and Understanding Magma Artefacts","doc":"The following sections contain background knowledge you need to be aware of, but which should NOT necessarily be covered in your response as it is documented elsewhere. Only mention absolutely necessary facts from it. Use a reference to the source if necessary.\n\n![[Magma.system.config#Context knowledge|]]\n\n##","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#context-knowledge"},{"type":"extras","title":"Description of the Magma project ![[Project#Description|]] - Creating and Understanding Magma Artefacts","doc":"![[Module.matter.config#Context knowledge|]]\n\n##","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#description-of-the-magma-project-project-description"},{"type":"extras","title":"Peripherally relevant modules - Creating and Understanding Magma Artefacts","doc":"##### `Some` ![[Some#Description|]]\n\n##### `Some.Example.Nested` ![[Some.Example.Nested#Description|]]\n\n![[ModuleDoc.artefact.config#Context knowledge|]]\n\n![[Some.Example#Context knowledge|]]","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#peripherally-relevant-modules"},{"type":"extras","title":"Request - Creating and Understanding Magma Artefacts","doc":"![[Some.Example#ModuleDoc prompt task|]]\n\n#","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#request"},{"type":"extras","title":"Description of the module `Some.Example` ![[Some.Example#Description|]] - Creating and Understanding Magma Artefacts","doc":"#","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#description-of-the-module-some-example-some-example-description"},{"type":"extras","title":"Module code - Creating and Understanding Magma Artefacts","doc":"This is the code of the module to be documented. Ignore commented out code.\n\n``elixir\ndefmodule Some.Module do\n  use Magma\n\n  # some code\t\nend\n``\n```\n\nBesides the project description, the \"Context knowledge\" section now transcludes the descriptions of all modules beneath the module to be documented  in a subsection \"Peripherally relevant modules\". For example, for a module `A.B.C`, the descriptions of the modules `A` and `A.B` are transcluded here. Also, all direct submodules are transcluded, i.e. in this case all modules `Some.Example.*`. If you prefer to transclude module descriptions on your own and want to circumvent possible duplicate transclusions, these automatic module context transclusions can be disabled via the `auto_module_context` property in the `Module.matter.config` document.\n\nAs can be seen in this artefact prompt, the \"Request\" section also includes the actual source code of the module to be documented. Use the \"Update\" button from the **Actions** in the prologue, to update the prompt after modifications of the code.\n\nThe process of executing the artefact prompt and choosing the final artefact is similar to the README artefact. However, that the artefact version can now be kept in a proper artefact version document (with respective YAML frontmatter) at the place where it belongs (in this case `artefacts/final/modules/Some/Module/ModuleDoc of Some.Module.md`) and without the need for a symbolic link.","ref":"magma-user-guide-creating-and-understanding-magma-artefacts-article-section.html#module-code"},{"type":"extras","title":"Generating Complex Artefacts","doc":"<!-- ExDoc doesn't support YAML frontmatter\n\n---\nmagma_type: Artefact.Version\nmagma_artefact: Article\nmagma_concept: \"[[Magma User Guide - Generating Complex Artefacts]]\"\nmagma_draft: \"[[Generated Magma User Guide - Generating Complex Artefacts (article section) (2023-11-01T13:21:49)]]\"\ncreated_at: 2023-11-01 13:29:41\ntags: [magma-vault]\naliases: []\n---\n\n-->\n\n# Generating Complex Artefacts\n\nThis section shows how Magma can be used to generate longer texts, such as user guides. It provides a detailed guide on how to use Magma for generating such complex artefacts composed of other artefacts.\n\nDue to the token limits of an LLM, generating lengthy texts can be challenging. Instead of generating complete texts all at once, Magma generates these texts section by section. The text generation process is modelled as follows:\n\n- Texts (`Magma.Matter.Text`) are a complex Magma matter type composed of various sections (`Magma.Matter.Text.Section`), which is also a Magma matter type.\n- The overall scope of the text is described in the concept document, which is used to generate a Table of Contents artefact. This artefact guides the generation of initial concept documents for the sections of the text.\n- The content of each section is described in its concept document. From these descriptions, parts of various artefacts such as an article, a presentation slide deck, or a screencast script can be generated. Currently, however, only the `Magma.Artefacts.Article` artefact type is implemented in Magma.\n- Finally, the final artefact version of the whole artefact version of a text is assembled from the artefact versions of the sections.\n\nThis might sound rather complex, but is in practice quite straightforward. Let's walk through the process of creating an article using Magma to see this in action.","ref":"magma-user-guide-generating-complex-artefacts-article-section.html"},{"type":"extras","title":"Creating Initial Documents - Generating Complex Artefacts","doc":"You can create the initial documents for a new text with the `Mix.Tasks.Magma.Text.New` Mix task. The first argument is the title of your text, followed by an optional text type:\n\n```sh\n$ mix magma.text.new \"Example User Guide\" UserGuide\n```\n\nThe text types determine the details of the system prompt of the artefact prompts. If no text type is provided, a minimal generic system prompt is used. Currently, there is only one text type predefined in this early stage of development, the `UserGuide` type. \n\n> ##","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#creating-initial-documents"},{"type":"extras","title":"Tip {: .tip} - Generating Complex Artefacts","doc":"> \n>With the `Mix.Tasks.Magma.Text.Type.New` Mix task, you can easily create your own text types. The given text type name must be a valid Elixir module name.\n> \n> ```sh\n> $ mix magma.text.type.new Book\n> ```\n> \n> This will create a new text type config document in the `magma.config/text_types` subdirectory of your vault, where you can define the system prompt and context knowledge for it. (If you created a generally useful text type, it would be nice to share it by opening a PR to add it to Magma. 🙏)\n\nThe following is an example of what the concept document looks like:\n\n```markdown\n---\nmagma_type: Concept\nmagma_matter_type: Text\nmagma_matter_text_type: UserGuide\ncreated_at: 2023-10-20 08:49:14\ntags: [magma-vault]\naliases: []\n---\n# Example User Guide","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#tip-tip"},{"type":"extras","title":"Description - Generating Complex Artefacts","doc":"<!--  \nWhat should \"Example User Guide\" cover?  \n-->\n\n\n# Context knowledge\n\n<!--  \nThis section should include background knowledge needed for the model to create a proper response, i.e. information it does not know either because of the knowledge cut-off date or unpublished knowledge.  \n  \nWrite it down right here in a subsection or use a transclusion. If applicable, specify source information that the model can use to generate a reference in the response.\n-->\n\n\n# Sections\n\n<!--  \nDon't remove or edit this section! The results of the generated table of contents will be copied to this place.  \n-->\n\n\n# Artefact previews\n\n-   [[Example User Guide (article) Preview]]\n\n\n# Artefacts","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#description"},{"type":"extras","title":"TableOfContents - Generating Complex Artefacts","doc":"-   Prompt: [[Prompt for Example User Guide ToC]]\n-   Final version: [[Example User Guide ToC]]\n\n#","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#tableofcontents"},{"type":"extras","title":"TableOfContents prompt task - Generating Complex Artefacts","doc":"Your task is to write an outline of \"Example User Guide\".\n\nPlease provide the outline in the following format:\n\n``markdown","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#tableofcontents-prompt-task"},{"type":"extras","title":"Title of the first section - Generating Complex Artefacts","doc":"Abstract: Abstract of the introduction.","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#title-of-the-first-section"},{"type":"extras","title":"Title of the next section - Generating Complex Artefacts","doc":"Abstract: Abstract of the next section.","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#title-of-the-next-section"},{"type":"extras","title":"Title of the another section - Generating Complex Artefacts","doc":"Abstract: Abstract of the another section.\n``\n\n<!--\nPlease don't change the general structure of this outline format. The section generator relies on an outline with sections.\n-->\n```\n\nAs you can see, there is a description section here as in every concept document. In this section, all content that should be included in the text should be roughly outlined. A detailed context knowledge base is also essential in this case.","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#title-of-the-another-section"},{"type":"extras","title":"Generating a Table of Content - Generating Complex Artefacts","doc":"In the \"Artefacts\" section, there is only a subsection for the generation of a table of contents (`Magma.Artefacts.TableOfContents` artefact type), which plays an important role in the following steps. From the generated table of contents artefact, the concept and artefact prompt documents of the individual sections are generated later. For this, it is important that the sections to be generated are specified in the produced artefact version of the table of contents as Markdown sections, which is pointed out by the comment at the end. However, as far as content in these sections is concerned, there are no specific requirements. The content generated by the LLM in these sections is used as the first content of the description of the concept of the respective section. Therefore, the generation of an abstract is requested here. However, the template can also be adapted in this respect if something different or additional is to be generated here.\n\nThe artefact prompt of the table of contents, its execution, and the creation of the artefact version are similar to the steps described in the previous chapters for generating other artefacts. The only notable difference for the prompts of artefacts about texts is that the transcluded system prompt is not coming from an artefact config document, but a config document for the text type, in this case the `UserGuide.text_type.config` document. This config document also defines a \"Context knowledge\" section which gets transcluded in the \"Context knowledge\" section of the prompt.","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#generating-a-table-of-content"},{"type":"extras","title":"Assembling Sections - Generating Complex Artefacts","doc":"Once the artefact version of the table of contents is generated, you can assemble the sections using the \"Assemble sections\" button or the `Mix.Tasks.Magma.Text.Assemble` Mix task:\n\n```sh\nmix magma.text.assemble \"Example User Guide ToC\"\n```\n\nThis task performs the following actions:\n\n-   It creates a concept document and an artefact prompt for each section of the article.\n-   It transcludes the descriptions of the sections in the \"Sections\" section of the concept document, providing an overview of the entire article.\n-   It creates a preview document for each artefact type of the text (currently only article). This document transcludes the artefact versions of all sections, allowing you to see a complete representation of the finished article at all times. It also serves as a basis for the final generation of the complete text by resolving the transclusions, which can be done using the \"Finalize\" button in the preview document or the `Mix.Tasks.Magma.Text.Finalize`:\n\n    ```sh\n    mix magma.text.finalize \"Example User Guide (article) Preview\"\n    ```","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#assembling-sections"},{"type":"extras","title":"Example - Generating Complex Artefacts","doc":"Let's illustrate this with another example. Suppose we want to create an article titled \"Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs\". Let's say after having the described what the text should be about in the concept, we came up with the following artefact version of the table of contents:\n\n```markdown\n---\nmagma_type: Artefact.Version\nmagma_artefact: TableOfContents\nmagma_concept: \"[[Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs]]\"\nmagma_draft: \"[[Generated Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs ToC (2023-10-25T22:31:55)]]\"\ncreated_at: 2023-10-25 22:31:33\ntags: [magma-vault]\naliases: []\n---\n\n``button\nname Assemble sections\ntype command\naction Shell commands: Execute: magma.text.assemble\ncolor blue\n``\n\n\n# Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs TOC","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#example"},{"type":"extras","title":"The Hurdles of Handling Voluminous Text - Generating Complex Artefacts","doc":"Abstract: Delving into the limitations of chat dialogs when managing extensive textual data. Issues such as scrolling challenges, lack of effective text segmentation, and limited editing capabilities come to the fore.","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#the-hurdles-of-handling-voluminous-text"},{"type":"extras","title":"Contextual Challenges and Fragmented Conversations - Generating Complex Artefacts","doc":"Abstract: Highlighting the issues of maintaining coherence and context in lengthy, intricate discussions within chat interfaces. This includes the frequent need for users to backtrack or reintroduce topics, leading to disjointed conversations.","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#contextual-challenges-and-fragmented-conversations"},{"type":"extras","title":"Rethinking the Interface - Generating Complex Artefacts","doc":"Abstract: Proposing alternative interfaces and modifications to address the identified challenges. From multi-pane designs to advanced organizational tools, a look into potential ways to enhance the experience of crafting complex texts with LLMs.\n```\n\nIf we hit the \"Assemble sections\" button, the \"Section\" section of our text concept document will be filled with this:\n\n```markdown\n# Sections\n\n## [[Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs - The Hurdles of Handling Voluminous Text|The Hurdles of Handling Voluminous Text]] ![[Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs - The Hurdles of Handling Voluminous Text#Description|]]\n\n## [[Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs - Contextual Challenges and Fragmented Conversations|Contextual Challenges and Fragmented Conversations]] ![[Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs - Contextual Challenges and Fragmented Conversations#Description|]]\n\n## [[Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs - Rethinking the Interface|Rethinking the Interface]] ![[Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs - Rethinking the Interface#Description|]]\n```\n\nThis looks quite overwhelming in this raw form, but it will look much cleaner when rendered in Obsidian: while the section titles link to the respective section concept documents, the content of the user written descriptions is transcluded.\n\nThe preview document looks very similar but transcludes the artefact versions of the sections instead.\n\nA generated concept document for a section would look like this:\n\n```markdown\n---\nmagma_type: Concept\nmagma_matter_type: Text.Section\nmagma_section_of: \"[[Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs]]\"\ncreated_at: 2023-10-25 22:42:35\ntags: [magma-vault]\naliases: []\n---\n# Rethinking the Interface","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#rethinking-the-interface"},{"type":"extras","title":"Description - Generating Complex Artefacts","doc":"Abstract: Proposing alternative interfaces and modifications to address the identified challenges. From multi-pane designs to advanced organizational tools, a look into potential ways to enhance the experience of crafting complex texts with LLMs.\n\n\n# Context knowledge\n\n<!--\nThis section should include background knowledge needed for the model to create a proper response, i.e. information it does not know either because of the knowledge cut-off date or unpublished knowledge.\n\nWrite it down right here in a subsection or use a transclusion. If applicable, specify source information that the model can use to generate a reference in the response.\n-->\n\n\n# Artefacts","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#description"},{"type":"extras","title":"Article - Generating Complex Artefacts","doc":"- Prompt: [[Prompt for Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs - Rethinking the Interface (article section)]]\n- Final version: [[Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs - Rethinking the Interface (article section)]]\n\n#","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#article"},{"type":"extras","title":"Article prompt task - Generating Complex Artefacts","doc":"Your task is to write the section \"Rethinking the Interface\" of \"Chat Dialogues and Complexity - The Mismatch in Crafting Elaborate Texts with LLMs\".\n\n```\n\nThe individual artefact versions of the sections can be generated in the usual way:\n\n1.  Write the concepts of each section, i.e., fill the \"Description\" section and compile the \"Context knowledge\". (Note: The content of the \"Context knowledge\" section of the concept of the whole text is transcluded in the respective artefact prompt of the section, i.e., only the section-specific \"Context knowledge\" of the respective section must be specified.)\n2.  Execute the prompt to generate different prompt results.\n3.  Select the best prompt result as the basis for the artefact version.\n4.  Edit the final version, which should now also be reviewed in the preview in the overall context.\n\nThis process ensures that you can generate complex artefacts in a structured and manageable way with Magma.","ref":"magma-user-guide-generating-complex-artefacts-article-section.html#article-prompt-task"},{"type":"extras","title":"Current Limitations and Roadmap","doc":"<!-- ExDoc doesn't support YAML frontmatter\n\n---\nmagma_type: Artefact.Version\nmagma_artefact: Article\nmagma_concept: \"[[Magma User Guide - Current Limitations and Roadmap]]\"\nmagma_draft: \"[[Generated Magma User Guide - Current Limitations and Roadmap (article section) (2023-11-01T17:23:14)]]\"\ncreated_at: 2023-11-01 17:28:03\ntags: [magma-vault]\naliases: []\n---\n\n-->\n\n# Current Limitations and Roadmap\n\nMagma is still in its early stages of development. While it already provides a useful environment for creating and executing complex prompts, it has some limitations and rough edges.","ref":"magma-user-guide-current-limitations-and-roadmap-article-section.html"},{"type":"extras","title":"Limitations - Current Limitations and Roadmap","doc":"- The shell commands triggered by the buttons on the various Magma documents are only tested under macOS. If you're experiencing problems under Linux or Windows, please report them on [this issue](https://github.com/marcelotto/magma/issues/1). If they work on one of these systems for you, please also confirm this on this issue.\n- The table of contents of a text cannot be easily modified after the initial section creation.\n- Intra-document transclusions (transclusions of sections inside the same document) are not supported due to a coarse transclusion recursion detection.\n- Previous artefact versions are not automatically backed up when selecting a new one, you'll have to use the Mix task to overwrite previous ones or delete them manually.\n- The moduledoc prompts produce mixed results and need further refinements.","ref":"magma-user-guide-current-limitations-and-roadmap-article-section.html#limitations"},{"type":"extras","title":"Roadmap - Current Limitations and Roadmap","doc":"Main goals for the next versions are:\n\n- Improving the process of changing the table of contents of a text after the initial section creation.\n- Supporting intra-document transclusions.\n- Implementing automatic backup for previous artefact versions when creating a new one.\n- Adding support for generating a project website.","ref":"magma-user-guide-current-limitations-and-roadmap-article-section.html#roadmap"},{"type":"extras","title":"Contributing to Magma - Current Limitations and Roadmap","doc":"Magma is an open-source project, and we welcome contributions from the community. If you'd like to help and contribute to the project, there's plenty of opportunities. Here are just some suggestions:\n\n- Test shell commands under Linux and Windows (see [this issue](https://github.com/marcelotto/magma/issues/1))\n- Simplifying the setup for non-Elixir users by creating a CLI and a self-contained binary with something like [Burrito](https://github.com/burrito-elixir/burrito).\n- Improving prompts.\n- Adding more text types.\n- Creating more artefacts for projects (like Project descriptions, Announcements for various sites/platforms) and modules (like Cheatsheets, Test factories, Test properties).\n- Adding more matter types like versions and tasks (stories, issues etc.).\n- Implementing support for module stereotypes (e.g. for GenServer, Mix tasks etc.) with more specific prompts.\n- Adding support for other LLMs (via `Magma.Generation` adapters).\n- Implementing support for other languages than Elixir.\n\nI'm excited about the future of Magma and hope you are too. Together, we can make Magma a powerful tool for developers and writers alike.","ref":"magma-user-guide-current-limitations-and-roadmap-article-section.html#contributing-to-magma"}],"content_type":"text/markdown"}